@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -66,6 +66,28 @@ static void _theme_list_dir(const gchar* const dir, GSList** result);
 static GString* _theme_find(const char* const theme_name);
 static gboolean _theme_load_file(const char* const theme_name);
 
+static void
+_theme_close(void)
+{
+    color_pair_cache_free();
+    if (theme) {
+        g_key_file_free(theme);
+        theme = NULL;
+    }
+    if (theme_loc) {
+        g_string_free(theme_loc, TRUE);
+        theme_loc = NULL;
+    }
+    if (bold_items) {
+        g_hash_table_destroy(bold_items);
+        bold_items = NULL;
+    }
+    if (defaults) {
+        g_hash_table_destroy(defaults);
+        defaults = NULL;
+    }
+}
+
 void
 theme_init(const char* const theme_name)
 {
@@ -77,6 +99,8 @@ theme_init(const char* const theme_name)
         }
     }
 
+    prof_add_shutdown_routine(_theme_close);
+
     defaults = g_hash_table_new_full(g_str_hash, g_str_equal, free, free);
 
     // Set default colors
@@ -248,27 +272,6 @@ theme_list(void)
     return result;
 }
 
-void
-theme_close(void)
-{
-    if (theme) {
-        g_key_file_free(theme);
-        theme = NULL;
-    }
-    if (theme_loc) {
-        g_string_free(theme_loc, TRUE);
-        theme_loc = NULL;
-    }
-    if (bold_items) {
-        g_hash_table_destroy(bold_items);
-        bold_items = NULL;
-    }
-    if (defaults) {
-        g_hash_table_destroy(defaults);
-        defaults = NULL;
-    }
-}
-
 void
 theme_init_colours(void)
 {
@@ -321,13 +324,10 @@ _load_preferences(void)
     _set_boolean_preference("roster.contacts", PREF_ROSTER_CONTACTS);
     _set_boolean_preference("roster.unsubscribed", PREF_ROSTER_UNSUBSCRIBED);
     _set_boolean_preference("roster.rooms", PREF_ROSTER_ROOMS);
-    _set_boolean_preference("roster.rooms.server", PREF_ROSTER_ROOMS_SERVER);
     _set_boolean_preference("privileges", PREF_MUC_PRIVILEGES);
     _set_boolean_preference("presence", PREF_PRESENCE);
     _set_boolean_preference("intype", PREF_INTYPE);
     _set_boolean_preference("enc.warn", PREF_ENC_WARN);
-    _set_boolean_preference("titlebar.muc.title.name", PREF_TITLEBAR_MUC_TITLE_NAME);
-    _set_boolean_preference("titlebar.muc.title.jid", PREF_TITLEBAR_MUC_TITLE_JID);
     _set_boolean_preference("tls.show", PREF_TLS_SHOW);
     _set_boolean_preference("statusbar.show.name", PREF_STATUSBAR_SHOW_NAME);
     _set_boolean_preference("statusbar.show.number", PREF_STATUSBAR_SHOW_NUMBER);
@@ -356,10 +356,11 @@ _load_preferences(void)
     _set_string_preference("roster.rooms.by", PREF_ROSTER_ROOMS_BY);
     _set_string_preference("roster.private", PREF_ROSTER_PRIVATE);
     _set_string_preference("roster.count", PREF_ROSTER_COUNT);
-    _set_string_preference("roster.rooms.use.name", PREF_ROSTER_ROOMS_USE_AS_NAME);
+    _set_string_preference("roster.rooms.title", PREF_ROSTER_ROOMS_TITLE);
     _set_string_preference("statusbar.self", PREF_STATUSBAR_SELF);
     _set_string_preference("statusbar.chat", PREF_STATUSBAR_CHAT);
-    _set_string_preference("statusbar.room", PREF_STATUSBAR_ROOM);
+    _set_string_preference("statusbar.room.title", PREF_STATUSBAR_ROOM_TITLE);
+    _set_string_preference("titlebar.muc.title", PREF_TITLEBAR_MUC_TITLE);
 
     // load ints from theme and set them to prefs
     // with custom set functions
@@ -486,8 +487,8 @@ _load_preferences(void)
 
     if (g_key_file_has_key(theme, "ui", "correction.char", NULL)) {
         auto_gchar gchar* ch = g_key_file_get_string(theme, "ui", "correction.char", NULL);
-        if (ch && strlen(ch) > 0) {
-            prefs_set_correction_char(ch[0]);
+        if (ch && g_utf8_strlen(ch, 4) == 1) {
+            prefs_set_correction_char(ch);
         }
     }
 
