@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2019 Paul Fariello <paul@fariello.eu>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -66,8 +66,6 @@
 #define AESGCM_URL_NONCE_LEN (2 * OMEMO_AESGCM_NONCE_LENGTH)
 #define AESGCM_URL_KEY_LEN   (2 * OMEMO_AESGCM_KEY_LENGTH)
 
-static gboolean loaded;
-
 static void _generate_pre_keys(int count);
 static void _generate_signed_pre_key(void);
 static gboolean _load_identity(void);
@@ -87,10 +85,8 @@ static void _acquire_sender_devices_list(void);
 
 typedef gboolean (*OmemoDeviceListHandler)(const char* const jid, GList* device_list);
 
-struct omemo_context_t
+typedef struct omemo_context
 {
-    pthread_mutexattr_t attr;
-    pthread_mutex_t lock;
     signal_context* signal;
     uint32_t device_id;
     GHashTable* device_list;
@@ -103,49 +99,55 @@ struct omemo_context_t
     GHashTable* pre_key_store;
     GHashTable* signed_pre_key_store;
     identity_key_store_t identity_key_store;
-    GString* identity_filename;
-    GKeyFile* identity_keyfile;
-    GString* trust_filename;
-    GKeyFile* trust_keyfile;
-    GString* sessions_filename;
-    GKeyFile* sessions_keyfile;
+    prof_keyfile_t identity;
+    prof_keyfile_t trust;
+    prof_keyfile_t sessions;
+    prof_keyfile_t knowndevices;
     GHashTable* known_devices;
-    GString* known_devices_filename;
-    GKeyFile* known_devices_keyfile;
-    GHashTable* fingerprint_ac;
-};
+    gboolean loaded;
+} omemo_context;
 
 static omemo_context omemo_ctx;
 
+static struct omemo_static_data
+{
+    pthread_mutexattr_t attr;
+    pthread_mutex_t lock;
+    GHashTable* fingerprint_ac;
+} omemo_static_data;
+
+static void
+_omemo_close(void)
+{
+    if (omemo_static_data.fingerprint_ac) {
+        g_hash_table_destroy(omemo_static_data.fingerprint_ac);
+        omemo_static_data.fingerprint_ac = NULL;
+    }
+    pthread_mutex_destroy(&omemo_static_data.lock);
+}
+
 void
 omemo_init(void)
 {
     log_info("[OMEMO] initialising");
-    if (omemo_crypto_init() != 0) {
-        cons_show("Error initializing OMEMO crypto: gcry_check_version() failed");
+
+    prof_add_shutdown_routine(_omemo_close);
+
+    gcry_error_t crypt_res = omemo_crypto_init();
+    if (crypt_res != 0) {
+        cons_show("Error initializing OMEMO crypto: %s", gcry_strerror(crypt_res));
     }
 
-    pthread_mutexattr_init(&omemo_ctx.attr);
-    pthread_mutexattr_settype(&omemo_ctx.attr, PTHREAD_MUTEX_RECURSIVE);
-    pthread_mutex_init(&omemo_ctx.lock, &omemo_ctx.attr);
+    pthread_mutexattr_init(&omemo_static_data.attr);
+    pthread_mutexattr_settype(&omemo_static_data.attr, PTHREAD_MUTEX_RECURSIVE);
+    pthread_mutex_init(&omemo_static_data.lock, &omemo_static_data.attr);
 
-    omemo_ctx.fingerprint_ac = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)autocomplete_free);
-}
-
-void
-omemo_close(void)
-{
-    if (omemo_ctx.fingerprint_ac) {
-        g_hash_table_destroy(omemo_ctx.fingerprint_ac);
-        omemo_ctx.fingerprint_ac = NULL;
-    }
+    omemo_static_data.fingerprint_ac = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)autocomplete_free);
 }
 
 void
 omemo_on_connect(ProfAccount* account)
 {
-    GError* error = NULL;
-
     if (signal_context_create(&omemo_ctx.signal, &omemo_ctx) != 0) {
         cons_show("Error initializing OMEMO context");
         return;
@@ -187,7 +189,7 @@ omemo_on_connect(ProfAccount* account)
         .contains_session_func = contains_session,
         .delete_session_func = delete_session,
         .delete_all_sessions_func = delete_all_sessions,
-        .destroy_func = NULL,
+        .destroy_func = (void (*)(void*))g_hash_table_destroy,
         .user_data = omemo_ctx.session_store
     };
     signal_protocol_store_context_set_session_store(omemo_ctx.store, &session_store);
@@ -198,7 +200,7 @@ omemo_on_connect(ProfAccount* account)
         .store_pre_key = store_pre_key,
         .contains_pre_key = contains_pre_key,
         .remove_pre_key = remove_pre_key,
-        .destroy_func = NULL,
+        .destroy_func = (void (*)(void*))g_hash_table_destroy,
         .user_data = omemo_ctx.pre_key_store
     };
     signal_protocol_store_context_set_pre_key_store(omemo_ctx.store, &pre_key_store);
@@ -209,7 +211,7 @@ omemo_on_connect(ProfAccount* account)
         .store_signed_pre_key = store_signed_pre_key,
         .contains_signed_pre_key = contains_signed_pre_key,
         .remove_signed_pre_key = remove_signed_pre_key,
-        .destroy_func = NULL,
+        .destroy_func = (void (*)(void*))g_hash_table_destroy,
         .user_data = omemo_ctx.signed_pre_key_store
     };
     signal_protocol_store_context_set_signed_pre_key_store(omemo_ctx.store, &signed_pre_key_store);
@@ -225,10 +227,10 @@ omemo_on_connect(ProfAccount* account)
     };
     signal_protocol_store_context_set_identity_key_store(omemo_ctx.store, &identity_key_store);
 
-    loaded = FALSE;
+    omemo_ctx.loaded = FALSE;
     omemo_ctx.device_list = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)g_list_free);
     omemo_ctx.device_list_handler = g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL);
-    omemo_ctx.known_devices = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)glib_hash_table_free);
+    omemo_ctx.known_devices = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)g_hash_table_destroy);
 
     auto_gchar gchar* omemo_dir = files_file_in_account_data_path(DIR_OMEMO, account->jid, NULL);
     if (!omemo_dir) {
@@ -236,88 +238,41 @@ omemo_on_connect(ProfAccount* account)
         return;
     }
 
-    omemo_ctx.identity_filename = g_string_new(omemo_dir);
-    g_string_append(omemo_ctx.identity_filename, "/identity.txt");
-    omemo_ctx.trust_filename = g_string_new(omemo_dir);
-    g_string_append(omemo_ctx.trust_filename, "/trust.txt");
-    omemo_ctx.sessions_filename = g_string_new(omemo_dir);
-    g_string_append(omemo_ctx.sessions_filename, "/sessions.txt");
-    omemo_ctx.known_devices_filename = g_string_new(omemo_dir);
-    g_string_append(omemo_ctx.known_devices_filename, "/known_devices.txt");
+    if (load_custom_keyfile(&omemo_ctx.identity, g_strdup_printf("%s/%s", omemo_dir, "identity.txt"))) {
+        if (!_load_identity())
+            return;
+    }
+
+    if (load_custom_keyfile(&omemo_ctx.trust, g_strdup_printf("%s/%s", omemo_dir, "trust.txt"))) {
+        _load_trust();
+    }
+
+    if (load_custom_keyfile(&omemo_ctx.sessions, g_strdup_printf("%s/%s", omemo_dir, "sessions.txt"))) {
+        _load_sessions();
+    }
+
+    if (load_custom_keyfile(&omemo_ctx.knowndevices, g_strdup_printf("%s/%s", omemo_dir, "known_devices.txt"))) {
+        _load_known_devices();
+    }
 
     omemo_devicelist_subscribe();
-
-    omemo_ctx.identity_keyfile = g_key_file_new();
-    omemo_ctx.trust_keyfile = g_key_file_new();
-    omemo_ctx.sessions_keyfile = g_key_file_new();
-    omemo_ctx.known_devices_keyfile = g_key_file_new();
-
-    if (g_key_file_load_from_file(omemo_ctx.identity_keyfile, omemo_ctx.identity_filename->str, G_KEY_FILE_KEEP_COMMENTS, &error)) {
-        if (!_load_identity()) {
-            return;
-        }
-    } else if (error->code != G_FILE_ERROR_NOENT) {
-        log_warning("[OMEMO] error loading identity from: %s, %s", omemo_ctx.identity_filename->str, error->message);
-        g_error_free(error);
-        return;
-    }
-
-    error = NULL;
-    if (g_key_file_load_from_file(omemo_ctx.trust_keyfile, omemo_ctx.trust_filename->str, G_KEY_FILE_KEEP_COMMENTS, &error)) {
-        _load_trust();
-    } else if (error->code != G_FILE_ERROR_NOENT) {
-        log_warning("[OMEMO] error loading trust from: %s, %s", omemo_ctx.trust_filename->str, error->message);
-        g_error_free(error);
-    } else {
-        log_warning("[OMEMO] no such file: %s", omemo_ctx.trust_filename->str);
-        g_error_free(error);
-    }
-
-    error = NULL;
-    if (g_key_file_load_from_file(omemo_ctx.sessions_keyfile, omemo_ctx.sessions_filename->str, G_KEY_FILE_KEEP_COMMENTS, &error)) {
-        _load_sessions();
-    } else if (error->code != G_FILE_ERROR_NOENT) {
-        log_warning("[OMEMO] error loading sessions from: %s, %s", omemo_ctx.sessions_filename->str, error->message);
-        g_error_free(error);
-    } else {
-        log_warning("[OMEMO] no such file: %s", omemo_ctx.trust_filename->str);
-        g_error_free(error);
-    }
-
-    error = NULL;
-    if (g_key_file_load_from_file(omemo_ctx.known_devices_keyfile, omemo_ctx.known_devices_filename->str, G_KEY_FILE_KEEP_COMMENTS, &error)) {
-        _load_known_devices();
-    } else if (error->code != G_FILE_ERROR_NOENT) {
-        log_warning("[OMEMO] error loading known devices from: %s, %s", omemo_ctx.known_devices_filename->str, error->message);
-        g_error_free(error);
-    } else {
-        log_warning("[OMEMO] no such file: %s", omemo_ctx.trust_filename->str);
-        g_error_free(error);
-    }
 }
 
 void
 omemo_on_disconnect(void)
 {
-    if (!loaded) {
+    if (!omemo_ctx.loaded) {
         return;
     }
 
-    glib_hash_table_free(omemo_ctx.signed_pre_key_store);
-    glib_hash_table_free(omemo_ctx.pre_key_store);
-    glib_hash_table_free(omemo_ctx.known_devices);
-    glib_hash_table_free(omemo_ctx.device_list_handler);
-    glib_hash_table_free(omemo_ctx.device_list);
+    g_hash_table_destroy(omemo_ctx.known_devices);
+    g_hash_table_destroy(omemo_ctx.device_list_handler);
+    g_hash_table_destroy(omemo_ctx.device_list);
 
-    g_string_free(omemo_ctx.identity_filename, TRUE);
-    g_key_file_free(omemo_ctx.identity_keyfile);
-    g_string_free(omemo_ctx.trust_filename, TRUE);
-    g_key_file_free(omemo_ctx.trust_keyfile);
-    g_string_free(omemo_ctx.sessions_filename, TRUE);
-    g_key_file_free(omemo_ctx.sessions_keyfile);
-    glib_hash_table_free(omemo_ctx.session_store);
-    g_string_free(omemo_ctx.known_devices_filename, TRUE);
-    g_key_file_free(omemo_ctx.known_devices_keyfile);
+    free_keyfile(&omemo_ctx.knowndevices);
+    free_keyfile(&omemo_ctx.sessions);
+    free_keyfile(&omemo_ctx.trust);
+    free_keyfile(&omemo_ctx.identity);
 
     signal_protocol_store_context_destroy(omemo_ctx.store);
     ec_public_key* pub = ratchet_identity_key_pair_get_public(omemo_ctx.identity_key_pair);
@@ -327,12 +282,13 @@ omemo_on_disconnect(void)
     ec_public_key_destroy((signal_type_base*)pub);
 
     signal_context_destroy(omemo_ctx.signal);
+    memset(&omemo_ctx, 0, sizeof(omemo_ctx));
 }
 
 void
 omemo_generate_crypto_materials(ProfAccount* account)
 {
-    if (loaded) {
+    if (omemo_ctx.loaded) {
         return;
     }
 
@@ -341,7 +297,7 @@ omemo_generate_crypto_materials(ProfAccount* account)
     /* Device ID */
     gcry_randomize(&omemo_ctx.device_id, 4, GCRY_VERY_STRONG_RANDOM);
     omemo_ctx.device_id &= 0x7fffffff;
-    g_key_file_set_uint64(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_DEVICE_ID, omemo_ctx.device_id);
+    g_key_file_set_uint64(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_DEVICE_ID, omemo_ctx.device_id);
     log_info("[OMEMO] device id: %d", omemo_ctx.device_id);
 
     /* Identity key */
@@ -349,15 +305,15 @@ omemo_generate_crypto_materials(ProfAccount* account)
 
     ec_public_key_serialize(&omemo_ctx.identity_key_store.public, ratchet_identity_key_pair_get_public(omemo_ctx.identity_key_pair));
     auto_gchar gchar* identity_key_public = g_base64_encode(signal_buffer_data(omemo_ctx.identity_key_store.public), signal_buffer_len(omemo_ctx.identity_key_store.public));
-    g_key_file_set_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PUBLIC, identity_key_public);
+    g_key_file_set_string(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PUBLIC, identity_key_public);
 
     ec_private_key_serialize(&omemo_ctx.identity_key_store.private, ratchet_identity_key_pair_get_private(omemo_ctx.identity_key_pair));
     auto_gchar gchar* identity_key_private = g_base64_encode(signal_buffer_data(omemo_ctx.identity_key_store.private), signal_buffer_len(omemo_ctx.identity_key_store.private));
-    g_key_file_set_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PRIVATE, identity_key_private);
+    g_key_file_set_string(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PRIVATE, identity_key_private);
 
     /* Registration ID */
     signal_protocol_key_helper_generate_registration_id(&omemo_ctx.registration_id, 0, omemo_ctx.signal);
-    g_key_file_set_uint64(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_REGISTRATION_ID, omemo_ctx.registration_id);
+    g_key_file_set_uint64(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_REGISTRATION_ID, omemo_ctx.registration_id);
 
     /* Pre keys */
     _generate_pre_keys(100);
@@ -367,7 +323,7 @@ omemo_generate_crypto_materials(ProfAccount* account)
 
     omemo_identity_keyfile_save();
 
-    loaded = TRUE;
+    omemo_ctx.loaded = TRUE;
 
     omemo_publish_crypto_materials();
     omemo_start_sessions();
@@ -378,7 +334,7 @@ omemo_publish_crypto_materials(void)
 {
     log_debug("[OMEMO] publish crypto materials");
 
-    if (loaded != TRUE) {
+    if (omemo_ctx.loaded != TRUE) {
         cons_show("OMEMO: cannot publish crypto materials before they are generated");
         log_error("[OMEMO] cannot publish crypto materials before they are generated");
         return;
@@ -390,7 +346,7 @@ omemo_publish_crypto_materials(void)
 static void
 _acquire_sender_devices_list(void)
 {
-    auto_char char* barejid = connection_get_barejid();
+    const char* barejid = connection_get_barejid();
 
     g_hash_table_insert(omemo_ctx.device_list_handler, strdup(barejid), _handle_own_device_list);
     omemo_devicelist_request(barejid);
@@ -426,8 +382,7 @@ omemo_start_session(const char* const barejid)
             log_debug("[OMEMO] missing device list for %s", barejid);
             // Own devices are handled by _handle_own_device_list
             // We won't add _handle_device_list_start_session for ourself
-            auto_char char* mybarejid = connection_get_barejid();
-            if (g_strcmp0(mybarejid, barejid) != 0) {
+            if (equals_our_barejid(barejid)) {
                 g_hash_table_insert(omemo_ctx.device_list_handler, strdup(barejid), _handle_device_list_start_session);
             }
             omemo_devicelist_request(barejid);
@@ -456,7 +411,7 @@ omemo_start_muc_sessions(const char* const roomjid)
 gboolean
 omemo_loaded(void)
 {
-    return loaded;
+    return omemo_ctx.loaded;
 }
 
 uint32_t
@@ -546,12 +501,7 @@ void
 omemo_set_device_list(const char* const from, GList* device_list)
 {
     log_debug("[OMEMO] Setting device list for %s", STR_MAYBE_NULL(from));
-    auto_jid Jid* jid;
-    if (from) {
-        jid = jid_create(from);
-    } else {
-        jid = jid_create(connection_get_fulljid());
-    }
+    auto_jid Jid* jid = jid_create(from ?: connection_get_fulljid());
 
     g_hash_table_insert(omemo_ctx.device_list, strdup(jid->barejid), device_list);
 
@@ -599,59 +549,43 @@ omemo_set_device_list(const char* const from, GList* device_list)
 GKeyFile*
 omemo_identity_keyfile(void)
 {
-    return omemo_ctx.identity_keyfile;
+    return omemo_ctx.identity.keyfile;
 }
 
 void
 omemo_identity_keyfile_save(void)
 {
-    GError* error = NULL;
-
-    if (!g_key_file_save_to_file(omemo_ctx.identity_keyfile, omemo_ctx.identity_filename->str, &error)) {
-        log_error("[OMEMO] error saving identity to: %s, %s", omemo_ctx.identity_filename->str, error->message);
-    }
+    save_keyfile(&omemo_ctx.identity);
 }
 
 GKeyFile*
 omemo_trust_keyfile(void)
 {
-    return omemo_ctx.trust_keyfile;
+    return omemo_ctx.trust.keyfile;
 }
 
 void
 omemo_trust_keyfile_save(void)
 {
-    GError* error = NULL;
-
-    if (!g_key_file_save_to_file(omemo_ctx.trust_keyfile, omemo_ctx.trust_filename->str, &error)) {
-        log_error("[OMEMO] error saving trust to: %s, %s", omemo_ctx.trust_filename->str, error->message);
-    }
+    save_keyfile(&omemo_ctx.trust);
 }
 
 GKeyFile*
 omemo_sessions_keyfile(void)
 {
-    return omemo_ctx.sessions_keyfile;
+    return omemo_ctx.sessions.keyfile;
 }
 
 void
 omemo_sessions_keyfile_save(void)
 {
-    GError* error = NULL;
-
-    if (!g_key_file_save_to_file(omemo_ctx.sessions_keyfile, omemo_ctx.sessions_filename->str, &error)) {
-        log_error("[OMEMO] error saving sessions to: %s, %s", omemo_ctx.sessions_filename->str, error->message);
-    }
+    save_keyfile(&omemo_ctx.sessions);
 }
 
 void
 omemo_known_devices_keyfile_save(void)
 {
-    GError* error = NULL;
-
-    if (!g_key_file_save_to_file(omemo_ctx.known_devices_keyfile, omemo_ctx.known_devices_filename->str, &error)) {
-        log_error("[OMEMO] error saving known devices to: %s, %s", omemo_ctx.known_devices_filename->str, error->message);
-    }
+    save_keyfile(&omemo_ctx.knowndevices);
 }
 
 void
@@ -694,7 +628,7 @@ omemo_start_device_session(const char* const jid, uint32_t device_id,
     }
 
     if (!contains_session(&jid_address, omemo_ctx.session_store)) {
-        log_debug("[OMEMO] There is no Session for %s ( %d) ,… building session.", jid_address.name, jid_address.device_id);
+        log_debug("[OMEMO] There is no Session for %s ( %d) ,? building session.", jid_address.name, jid_address.device_id);
         int res;
 
         address = malloc(sizeof(signal_protocol_address));
@@ -752,7 +686,7 @@ omemo_on_message_send(ProfWin* win, const char* const message, gboolean request_
 {
     char* id = NULL;
     int res;
-    auto_jid Jid* jid = jid_create(connection_get_fulljid());
+    const Jid* jid = connection_get_jid();
     GList* keys = NULL;
 
     unsigned char* key;
@@ -826,15 +760,12 @@ omemo_on_message_send(ProfWin* win, const char* const message, gboolean request_
             // Don't encrypt for this device (according to
             // <https://xmpp.org/extensions/xep-0384.html#encrypt>).
             // Yourself as recipients in case of MUC
-            char* mybarejid = connection_get_barejid();
-            if (!g_strcmp0(mybarejid, recipients_iter->data)) {
+            if (equals_our_barejid(recipients_iter->data)) {
                 if (GPOINTER_TO_INT(device_ids_iter->data) == omemo_ctx.device_id) {
-                    free(mybarejid);
                     log_debug("[OMEMO][SEND] Skipping %d (my device) ", GPOINTER_TO_INT(device_ids_iter->data));
                     continue;
                 }
             }
-            free(mybarejid);
 
             log_debug("[OMEMO][SEND] recipients with device id %d for %s", GPOINTER_TO_INT(device_ids_iter->data), recipients_iter->data);
             res = session_cipher_create(&cipher, omemo_ctx.store, &address, omemo_ctx.signal);
@@ -1330,22 +1261,20 @@ omemo_untrust(const char* const jid, const char* const fingerprint_formatted)
 
     /* Remove from keyfile */
     auto_gchar gchar* device_id_str = g_strdup_printf("%d", device_id);
-    g_key_file_remove_key(omemo_ctx.trust_keyfile, jid, device_id_str, NULL);
+    g_key_file_remove_key(omemo_ctx.trust.keyfile, jid, device_id_str, NULL);
     omemo_trust_keyfile_save();
 }
 
 static void
 _lock(void* user_data)
 {
-    omemo_context* ctx = (omemo_context*)user_data;
-    pthread_mutex_lock(&ctx->lock);
+    pthread_mutex_lock(&omemo_static_data.lock);
 }
 
 static void
 _unlock(void* user_data)
 {
-    omemo_context* ctx = (omemo_context*)user_data;
-    pthread_mutex_unlock(&ctx->lock);
+    pthread_mutex_unlock(&omemo_static_data.lock);
 }
 
 static void
@@ -1413,7 +1342,7 @@ omemo_key_free(omemo_key_t* key)
 char*
 omemo_fingerprint_autocomplete(const char* const search_str, gboolean previous, void* context)
 {
-    Autocomplete ac = g_hash_table_lookup(omemo_ctx.fingerprint_ac, context);
+    Autocomplete ac = g_hash_table_lookup(omemo_static_data.fingerprint_ac, context);
     if (ac != NULL) {
         return autocomplete_complete(ac, search_str, FALSE, previous);
     } else {
@@ -1424,9 +1353,11 @@ omemo_fingerprint_autocomplete(const char* const search_str, gboolean previous,
 void
 omemo_fingerprint_autocomplete_reset(void)
 {
+    if (!omemo_static_data.fingerprint_ac)
+        return;
     gpointer value;
     GHashTableIter iter;
-    g_hash_table_iter_init(&iter, omemo_ctx.fingerprint_ac);
+    g_hash_table_iter_init(&iter, omemo_static_data.fingerprint_ac);
 
     while (g_hash_table_iter_next(&iter, NULL, &value)) {
         Autocomplete ac = value;
@@ -1438,8 +1369,7 @@ gboolean
 omemo_automatic_start(const char* const recipient)
 {
     gboolean result = FALSE;
-    char* account_name = session_get_account_name();
-    ProfAccount* account = accounts_get_account(account_name);
+    ProfAccount* account = accounts_get_account(session_get_account_name());
     prof_omemopolicy_t policy;
 
     if (account->omemo_policy) {
@@ -1501,7 +1431,7 @@ _load_identity(void)
 
     /* Device ID */
     error = NULL;
-    omemo_ctx.device_id = g_key_file_get_uint64(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_DEVICE_ID, &error);
+    omemo_ctx.device_id = g_key_file_get_uint64(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_DEVICE_ID, &error);
     if (error != NULL) {
         log_error("[OMEMO] cannot load device id: %s", error->message);
         return FALSE;
@@ -1510,7 +1440,7 @@ _load_identity(void)
 
     /* Registration ID */
     error = NULL;
-    omemo_ctx.registration_id = g_key_file_get_uint64(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_REGISTRATION_ID, &error);
+    omemo_ctx.registration_id = g_key_file_get_uint64(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_REGISTRATION_ID, &error);
     if (error != NULL) {
         log_error("[OMEMO] cannot load registration id: %s", error->message);
         return FALSE;
@@ -1518,7 +1448,7 @@ _load_identity(void)
 
     /* Identity key */
     error = NULL;
-    auto_gchar gchar* identity_key_public_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PUBLIC, &error);
+    auto_gchar gchar* identity_key_public_b64 = g_key_file_get_string(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PUBLIC, &error);
     if (!identity_key_public_b64) {
         log_error("[OMEMO] cannot load identity public key: %s", error->message);
         return FALSE;
@@ -1529,7 +1459,7 @@ _load_identity(void)
     omemo_ctx.identity_key_store.public = signal_buffer_create(identity_key_public, identity_key_public_len);
 
     error = NULL;
-    auto_gchar gchar* identity_key_private_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PRIVATE, &error);
+    auto_gchar gchar* identity_key_private_b64 = g_key_file_get_string(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PRIVATE, &error);
     if (!identity_key_private_b64) {
         log_error("[OMEMO] cannot load identity private key: %s", error->message);
         return FALSE;
@@ -1549,10 +1479,10 @@ _load_identity(void)
     int i;
     /* Pre keys */
     i = 0;
-    keys = g_key_file_get_keys(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_PREKEYS, NULL, NULL);
+    keys = g_key_file_get_keys(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_PREKEYS, NULL, NULL);
     if (keys) {
         for (i = 0; keys[i] != NULL; i++) {
-            auto_gchar gchar* pre_key_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_PREKEYS, keys[i], NULL);
+            auto_gchar gchar* pre_key_b64 = g_key_file_get_string(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_PREKEYS, keys[i], NULL);
             size_t pre_key_len;
             auto_guchar guchar* pre_key = g_base64_decode(pre_key_b64, &pre_key_len);
             signal_buffer* buffer = signal_buffer_create(pre_key, pre_key_len);
@@ -1569,10 +1499,10 @@ _load_identity(void)
 
     /* Signed pre keys */
     i = 0;
-    keys = g_key_file_get_keys(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_SIGNED_PREKEYS, NULL, NULL);
+    keys = g_key_file_get_keys(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_SIGNED_PREKEYS, NULL, NULL);
     if (keys) {
         for (i = 0; keys[i] != NULL; i++) {
-            auto_gchar gchar* signed_pre_key_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_SIGNED_PREKEYS, keys[i], NULL);
+            auto_gchar gchar* signed_pre_key_b64 = g_key_file_get_string(omemo_ctx.identity.keyfile, OMEMO_STORE_GROUP_SIGNED_PREKEYS, keys[i], NULL);
             size_t signed_pre_key_len;
             auto_guchar guchar* signed_pre_key = g_base64_decode(signed_pre_key_b64, &signed_pre_key_len);
             signal_buffer* buffer = signal_buffer_create(signed_pre_key, signed_pre_key_len);
@@ -1586,7 +1516,7 @@ _load_identity(void)
         _generate_signed_pre_key();
     }
 
-    loaded = TRUE;
+    omemo_ctx.loaded = TRUE;
 
     omemo_identity_keyfile_save();
     omemo_start_sessions();
@@ -1597,7 +1527,7 @@ _load_identity(void)
 static void
 _load_trust(void)
 {
-    auto_gcharv gchar** groups = g_key_file_get_groups(omemo_ctx.trust_keyfile, NULL);
+    auto_gcharv gchar** groups = g_key_file_get_groups(omemo_ctx.trust.keyfile, NULL);
 
     if (!groups) {
         return;
@@ -1612,9 +1542,9 @@ _load_trust(void)
             g_hash_table_insert(omemo_ctx.identity_key_store.trusted, strdup(groups[i]), trusted);
         }
 
-        auto_gcharv gchar** keys = g_key_file_get_keys(omemo_ctx.trust_keyfile, groups[i], NULL, NULL);
+        auto_gcharv gchar** keys = g_key_file_get_keys(omemo_ctx.trust.keyfile, groups[i], NULL, NULL);
         for (int j = 0; keys[j] != NULL; j++) {
-            auto_gchar gchar* key_b64 = g_key_file_get_string(omemo_ctx.trust_keyfile, groups[i], keys[j], NULL);
+            auto_gchar gchar* key_b64 = g_key_file_get_string(omemo_ctx.trust.keyfile, groups[i], keys[j], NULL);
             size_t key_len;
             auto_guchar guchar* key = g_base64_decode(key_b64, &key_len);
             signal_buffer* buffer = signal_buffer_create(key, key_len);
@@ -1627,7 +1557,7 @@ _load_trust(void)
 static void
 _load_sessions(void)
 {
-    auto_gcharv gchar** groups = g_key_file_get_groups(omemo_ctx.sessions_keyfile, NULL);
+    auto_gcharv gchar** groups = g_key_file_get_groups(omemo_ctx.sessions.keyfile, NULL);
 
     if (!groups) {
         return;
@@ -1642,10 +1572,10 @@ _load_sessions(void)
             g_hash_table_insert(omemo_ctx.session_store, strdup(groups[i]), device_store);
         }
 
-        auto_gcharv gchar** keys = g_key_file_get_keys(omemo_ctx.sessions_keyfile, groups[i], NULL, NULL);
+        auto_gcharv gchar** keys = g_key_file_get_keys(omemo_ctx.sessions.keyfile, groups[i], NULL, NULL);
         for (int j = 0; keys[j] != NULL; j++) {
             uint32_t id = strtoul(keys[j], NULL, 10);
-            auto_gchar gchar* record_b64 = g_key_file_get_string(omemo_ctx.sessions_keyfile, groups[i], keys[j], NULL);
+            auto_gchar gchar* record_b64 = g_key_file_get_string(omemo_ctx.sessions.keyfile, groups[i], keys[j], NULL);
             size_t record_len;
             auto_guchar guchar* record = g_base64_decode(record_b64, &record_len);
             signal_buffer* buffer = signal_buffer_create(record, record_len);
@@ -1657,7 +1587,7 @@ _load_sessions(void)
 static void
 _load_known_devices(void)
 {
-    auto_gcharv gchar** groups = g_key_file_get_groups(omemo_ctx.known_devices_keyfile, NULL);
+    auto_gcharv gchar** groups = g_key_file_get_groups(omemo_ctx.knowndevices.keyfile, NULL);
 
     if (!groups) {
         return;
@@ -1672,10 +1602,10 @@ _load_known_devices(void)
             g_hash_table_insert(omemo_ctx.known_devices, strdup(groups[i]), known_identities);
         }
 
-        auto_gcharv gchar** keys = g_key_file_get_keys(omemo_ctx.known_devices_keyfile, groups[i], NULL, NULL);
+        auto_gcharv gchar** keys = g_key_file_get_keys(omemo_ctx.knowndevices.keyfile, groups[i], NULL, NULL);
         for (int j = 0; keys[j] != NULL; j++) {
             uint32_t device_id = strtoul(keys[j], NULL, 10);
-            auto_gchar gchar* fingerprint = g_key_file_get_string(omemo_ctx.known_devices_keyfile, groups[i], keys[j], NULL);
+            auto_gchar gchar* fingerprint = g_key_file_get_string(omemo_ctx.knowndevices.keyfile, groups[i], keys[j], NULL);
             g_hash_table_insert(known_identities, strdup(fingerprint), GINT_TO_POINTER(device_id));
         }
     }
@@ -1695,13 +1625,13 @@ _cache_device_identity(const char* const jid, uint32_t device_id, ec_public_key*
     g_hash_table_insert(known_identities, strdup(fingerprint), GINT_TO_POINTER(device_id));
 
     auto_gchar gchar* device_id_str = g_strdup_printf("%d", device_id);
-    g_key_file_set_string(omemo_ctx.known_devices_keyfile, jid, device_id_str, fingerprint);
+    g_key_file_set_string(omemo_ctx.knowndevices.keyfile, jid, device_id_str, fingerprint);
     omemo_known_devices_keyfile_save();
 
-    Autocomplete ac = g_hash_table_lookup(omemo_ctx.fingerprint_ac, jid);
+    Autocomplete ac = g_hash_table_lookup(omemo_static_data.fingerprint_ac, jid);
     if (ac == NULL) {
         ac = autocomplete_new();
-        g_hash_table_insert(omemo_ctx.fingerprint_ac, strdup(jid), ac);
+        g_hash_table_insert(omemo_static_data.fingerprint_ac, strdup(jid), ac);
     }
 
     char* formatted_fingerprint = omemo_format_fingerprint(fingerprint);
@@ -1878,14 +1808,10 @@ out:
 char*
 omemo_qrcode_str()
 {
-    char* mybarejid = connection_get_barejid();
-    char* fingerprint = omemo_own_fingerprint(FALSE);
+    auto_char char* fingerprint = omemo_own_fingerprint(FALSE);
     uint32_t sid = omemo_device_id();
 
-    char* qrstr = g_strdup_printf("xmpp:%s?omemo-sid-%d=%s", mybarejid, sid, fingerprint);
-
-    free(mybarejid);
-    free(fingerprint);
+    char* qrstr = g_strdup_printf("xmpp:%s?omemo-sid-%d=%s", connection_get_barejid(), sid, fingerprint);
 
     return qrstr;
 }
