@@ -122,15 +122,14 @@ static void
 cb_inject_message(void* opdata, const char* accountname,
                   const char* protocol, const char* recipient, const char* message)
 {
-    char* id = message_send_chat_otr(recipient, message, FALSE, NULL);
-    free(id);
+    free(message_send_chat_otr(recipient, message, FALSE, NULL));
 }
 
 static void
 cb_write_fingerprints(void* opdata)
 {
     gcry_error_t err = 0;
-    gchar* fpsfilename = files_file_in_account_data_path(DIR_OTR, jid, "fingerprints.txt");
+    auto_gchar gchar* fpsfilename = files_file_in_account_data_path(DIR_OTR, jid, "fingerprints.txt");
     if (!fpsfilename) {
         log_error("Failed to create fingerprints file");
         cons_show_error("Failed to create fingerprints file");
@@ -142,8 +141,6 @@ cb_write_fingerprints(void* opdata)
         log_error("Failed to write fingerprints file");
         cons_show_error("Failed to write fingerprints file");
     }
-
-    g_free(fpsfilename);
 }
 
 static void
@@ -151,7 +148,7 @@ cb_gone_secure(void* opdata, ConnContext* context)
 {
     ProfChatWin* chatwin = wins_get_chat(context->username);
     if (!chatwin) {
-        chatwin = (ProfChatWin*)wins_new_chat(context->username);
+        chatwin = chatwin_new(context->username);
     }
 
     chatwin_otr_secured(chatwin, otr_is_trusted(context->username));
@@ -169,12 +166,29 @@ otr_start_query(void)
     return otrlib_start_query();
 }
 
+static void
+_otr_shutdown(void)
+{
+    g_hash_table_destroy(smp_initiators);
+    if (jid) {
+        free(jid);
+        jid = NULL;
+    }
+    if (user_state) {
+        otrl_userstate_free(user_state);
+        user_state = NULL;
+    }
+    otrlib_shutdown();
+}
+
 void
 otr_init(void)
 {
     log_info("Initialising OTR");
     OTRL_INIT;
 
+    prof_add_shutdown_routine(_otr_shutdown);
+
     jid = NULL;
 
     ops.policy = cb_policy;
@@ -190,15 +204,6 @@ otr_init(void)
     data_loaded = FALSE;
 }
 
-void
-otr_shutdown(void)
-{
-    if (jid) {
-        free(jid);
-        jid = NULL;
-    }
-}
-
 void
 otr_poll(void)
 {
@@ -214,7 +219,7 @@ otr_on_connect(ProfAccount* account)
     jid = strdup(account->jid);
     log_info("Loading OTR key for %s", jid);
 
-    gchar* otr_dir = files_file_in_account_data_path(DIR_OTR, jid, NULL);
+    auto_gchar gchar* otr_dir = files_file_in_account_data_path(DIR_OTR, jid, NULL);
     if (!otr_dir) {
         log_error("Could not create directory for account %s.", jid);
         cons_show_error("Could not create directory for account %s.", jid);
@@ -239,7 +244,6 @@ otr_on_connect(ProfAccount* account)
         if (err != GPG_ERR_NO_ERROR) {
             log_warning("Failed to read OTR private key file: %s", keysfilename->str);
             cons_show_error("Failed to read OTR private key file: %s", keysfilename->str);
-            g_free(otr_dir);
             g_string_free(keysfilename, TRUE);
             return;
         }
@@ -248,7 +252,6 @@ otr_on_connect(ProfAccount* account)
         if (!privkey) {
             log_warning("No OTR private key found for account \"%s\", protocol \"xmpp\" in file: %s", jid, keysfilename->str);
             cons_show_error("No OTR private key found for account \"%s\", protocol \"xmpp\" in file: %s", jid, keysfilename->str);
-            g_free(otr_dir);
             g_string_free(keysfilename, TRUE);
             return;
         }
@@ -266,7 +269,6 @@ otr_on_connect(ProfAccount* account)
         err = otrl_privkey_read_fingerprints(user_state, fpsfilename->str, NULL, NULL);
         if (err != GPG_ERR_NO_ERROR) {
             log_error("Failed to load OTR fingerprints file: %s", fpsfilename->str);
-            g_free(otr_dir);
             g_string_free(keysfilename, TRUE);
             g_string_free(fpsfilename, TRUE);
             return;
@@ -280,7 +282,6 @@ otr_on_connect(ProfAccount* account)
         cons_show("Loaded OTR private key for %s", jid);
     }
 
-    g_free(otr_dir);
     g_string_free(keysfilename, TRUE);
     g_string_free(fpsfilename, TRUE);
     return;
@@ -304,9 +305,8 @@ otr_on_message_recv(const char* const barejid, const char* const resource, const
                 }
                 memmove(whitespace_base, whitespace_base + tag_length, tag_length);
                 char* otr_query_message = otr_start_query();
-                cons_show("OTR Whitespace pattern detected. Attempting to start OTR session...");
-                char* id = message_send_chat_otr(barejid, otr_query_message, FALSE, NULL);
-                free(id);
+                cons_show("OTR Whitespace pattern detected. Attempting to start OTR session…");
+                free(message_send_chat_otr(barejid, otr_query_message, FALSE, NULL));
             }
         }
     }
@@ -318,9 +318,8 @@ otr_on_message_recv(const char* const barejid, const char* const resource, const
 
     if (policy == PROF_OTRPOLICY_ALWAYS && *decrypted == FALSE && !whitespace_base) {
         char* otr_query_message = otr_start_query();
-        cons_show("Attempting to start OTR session...");
-        char* id = message_send_chat_otr(barejid, otr_query_message, FALSE, NULL);
-        free(id);
+        cons_show("Attempting to start OTR session…");
+        free(message_send_chat_otr(barejid, otr_query_message, FALSE, NULL));
     }
 
     return newmessage;
@@ -329,7 +328,7 @@ otr_on_message_recv(const char* const barejid, const char* const resource, const
 gboolean
 otr_on_message_send(ProfChatWin* chatwin, const char* const message, gboolean request_receipt, const char* const replace_id)
 {
-    char* id = NULL;
+    auto_char char* id = NULL;
     prof_otrpolicy_t policy = otr_get_policy(chatwin->barejid);
 
     // Send encrypted message
@@ -341,7 +340,6 @@ otr_on_message_send(ProfChatWin* chatwin, const char* const message, gboolean re
             log_database_add_outgoing_chat(id, chatwin->barejid, message, replace_id, PROF_MSG_ENC_OTR);
             chatwin_outgoing_msg(chatwin, message, id, PROF_MSG_ENC_OTR, request_receipt, replace_id);
             otr_free_message(encrypted);
-            free(id);
             return TRUE;
         } else {
             win_println((ProfWin*)chatwin, THEME_ERROR, "-", "%s", "Failed to encrypt and send message.");
@@ -357,12 +355,10 @@ otr_on_message_send(ProfChatWin* chatwin, const char* const message, gboolean re
 
     // tag and send for policy opportunistic
     if (policy == PROF_OTRPOLICY_OPPORTUNISTIC) {
-        char* otr_tagged_msg = otr_tag_message(message);
+        auto_char char* otr_tagged_msg = otr_tag_message(message);
         id = message_send_chat_otr(chatwin->barejid, otr_tagged_msg, request_receipt, replace_id);
         chatwin_outgoing_msg(chatwin, message, id, PROF_MSG_ENC_NONE, request_receipt, replace_id);
         chat_log_msg_out(chatwin->barejid, message, NULL);
-        free(otr_tagged_msg);
-        free(id);
         return TRUE;
     }
 
@@ -381,7 +377,7 @@ otr_keygen(ProfAccount* account)
     jid = strdup(account->jid);
     log_info("Generating OTR key for %s", jid);
 
-    gchar* otr_dir = files_file_in_account_data_path(DIR_OTR, jid, NULL);
+    auto_gchar gchar* otr_dir = files_file_in_account_data_path(DIR_OTR, jid, NULL);
 
     if (!otr_dir) {
         log_error("Could not create directory for account %s.", jid);
@@ -399,7 +395,6 @@ otr_keygen(ProfAccount* account)
     ui_update();
     err = otrl_privkey_generate(user_state, keysfilename->str, account->jid, "xmpp");
     if (err != GPG_ERR_NO_ERROR) {
-        g_free(otr_dir);
         g_string_free(keysfilename, TRUE);
         log_error("Failed to generate private key");
         cons_show_error("Failed to generate private key");
@@ -414,7 +409,6 @@ otr_keygen(ProfAccount* account)
     log_debug("Generating fingerprints file %s for %s", fpsfilename->str, jid);
     err = otrl_privkey_write_fingerprints(user_state, fpsfilename->str);
     if (err != GPG_ERR_NO_ERROR) {
-        g_free(otr_dir);
         g_string_free(keysfilename, TRUE);
         log_error("Failed to create fingerprints file");
         cons_show_error("Failed to create fingerprints file");
@@ -424,7 +418,6 @@ otr_keygen(ProfAccount* account)
 
     err = otrl_privkey_read(user_state, keysfilename->str);
     if (err != GPG_ERR_NO_ERROR) {
-        g_free(otr_dir);
         g_string_free(keysfilename, TRUE);
         log_error("Failed to load private key");
         data_loaded = FALSE;
@@ -433,7 +426,6 @@ otr_keygen(ProfAccount* account)
 
     err = otrl_privkey_read_fingerprints(user_state, fpsfilename->str, NULL, NULL);
     if (err != GPG_ERR_NO_ERROR) {
-        g_free(otr_dir);
         g_string_free(keysfilename, TRUE);
         log_error("Failed to load fingerprints");
         data_loaded = FALSE;
@@ -442,7 +434,6 @@ otr_keygen(ProfAccount* account)
 
     data_loaded = TRUE;
 
-    g_free(otr_dir);
     g_string_free(keysfilename, TRUE);
     g_string_free(fpsfilename, TRUE);
     return;
@@ -457,13 +448,7 @@ otr_key_loaded(void)
 char*
 otr_tag_message(const char* const msg)
 {
-    GString* otr_message = g_string_new(msg);
-    g_string_append(otr_message, OTRL_MESSAGE_TAG_BASE);
-    g_string_append(otr_message, OTRL_MESSAGE_TAG_V2);
-    char* result = otr_message->str;
-    g_string_free(otr_message, FALSE);
-
-    return result;
+    return g_strdup_printf("%s%s%s", msg, OTRL_MESSAGE_TAG_BASE, OTRL_MESSAGE_TAG_V2);
 }
 
 gboolean
@@ -656,8 +641,7 @@ otr_get_their_fingerprint(const char* const recipient)
 prof_otrpolicy_t
 otr_get_policy(const char* const recipient)
 {
-    char* account_name = session_get_account_name();
-    ProfAccount* account = accounts_get_account(account_name);
+    ProfAccount* account = accounts_get_account(session_get_account_name());
     // check contact specific setting
     if (g_list_find_custom(account->otr_manual, recipient, (GCompareFunc)g_strcmp0)) {
         account_free(account);
@@ -690,7 +674,7 @@ otr_get_policy(const char* const recipient)
     account_free(account);
 
     // check global setting
-    char* pref_otr_policy = prefs_get_string(PREF_OTR_POLICY);
+    auto_gchar gchar* pref_otr_policy = prefs_get_string(PREF_OTR_POLICY);
 
     // pref defaults to manual
     prof_otrpolicy_t result = PROF_OTRPOLICY_MANUAL;
@@ -701,8 +685,6 @@ otr_get_policy(const char* const recipient)
         result = PROF_OTRPOLICY_ALWAYS;
     }
 
-    g_free(pref_otr_policy);
-
     return result;
 }
 
