@@ -62,6 +62,12 @@ otrlib_init_timer(void)
     current_interval = otrl_message_poll_get_default_interval(user_state);
 }
 
+void
+otrlib_shutdown(void)
+{
+    g_timer_destroy(timer);
+}
+
 void
 otrlib_poll(void)
 {
