@@ -65,9 +65,49 @@
 
 static GHashTable* plugins;
 
+static void
+_plugins_shutdown(void)
+{
+    GList* values = g_hash_table_get_values(plugins);
+    GList *curr = values, *next;
+
+    while (curr) {
+        next = g_list_next(curr);
+#ifdef HAVE_PYTHON
+        if (curr && ((ProfPlugin*)curr->data)->lang == LANG_PYTHON) {
+            python_plugin_destroy(curr->data);
+            curr = NULL;
+        }
+#endif
+#ifdef HAVE_C
+        if (curr && ((ProfPlugin*)curr->data)->lang == LANG_C) {
+            c_plugin_destroy(curr->data);
+            curr = NULL;
+        }
+#endif
+        curr = next;
+    }
+    g_list_free(values);
+#ifdef HAVE_PYTHON
+    python_shutdown();
+#endif
+#ifdef HAVE_C
+    c_shutdown();
+#endif
+
+    plugin_settings_close();
+    plugin_themes_close();
+    autocompleters_destroy();
+    callbacks_close();
+    disco_close();
+    g_hash_table_destroy(plugins);
+    plugins = NULL;
+}
+
 void
 plugins_init(void)
 {
+    prof_add_shutdown_routine(_plugins_shutdown);
     plugins = g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL);
     callbacks_init();
     autocompleters_init();
@@ -252,9 +292,7 @@ plugins_load(const char* const name, GString* error_message)
     if (plugin) {
         g_hash_table_insert(plugins, strdup(name), plugin);
         if (connection_get_status() == JABBER_CONNECTED) {
-            const char* account_name = session_get_account_name();
-            const char* fulljid = connection_get_fulljid();
-            plugin->init_func(plugin, PACKAGE_VERSION, PACKAGE_STATUS, account_name, fulljid);
+            plugin->init_func(plugin, PACKAGE_VERSION, PACKAGE_STATUS, session_get_account_name(), connection_get_fulljid());
         } else {
             plugin->init_func(plugin, PACKAGE_VERSION, PACKAGE_STATUS, NULL, NULL);
         }
@@ -315,8 +353,7 @@ plugins_unload(const char* const name)
         caps_reset_ver();
         // resend presence to update server's disco info data for this client
         if (connection_get_status() == JABBER_CONNECTED) {
-            char* account_name = session_get_account_name();
-            resource_presence_t last_presence = accounts_get_last_presence(account_name);
+            resource_presence_t last_presence = accounts_get_last_presence(session_get_account_name());
             cl_ev_presence_send(last_presence, 0);
         }
         return TRUE;
@@ -354,12 +391,15 @@ plugins_reload(const char* const name, GString* error_message)
     gboolean res = plugins_unload(name);
     if (res) {
         res = plugins_load(name, error_message);
+    } else {
+        log_info("Failed to reload plugin: %s, not loaded", name);
+        g_string_assign(error_message, "cannot unload");
     }
 
     return res;
 }
 
-void
+static void
 _plugins_unloaded_list_dir(const gchar* const dir, GSList** result)
 {
     GDir* plugins_dir = g_dir_open(dir, 0, NULL);
@@ -421,21 +461,8 @@ plugins_close_win(const char* const plugin_name, const char* const tag)
     callbacks_remove_win(plugin_name, tag);
 }
 
-void
-plugins_on_start(void)
-{
-    GList* values = g_hash_table_get_values(plugins);
-    GList* curr = values;
-    while (curr) {
-        ProfPlugin* plugin = curr->data;
-        plugin->on_start_func(plugin);
-        curr = g_list_next(curr);
-    }
-    g_list_free(values);
-}
-
-void
-plugins_on_shutdown(void)
+static void
+_plugins_on_shutdown(void)
 {
     GList* values = g_hash_table_get_values(plugins);
     GList* curr = values;
@@ -447,6 +474,20 @@ plugins_on_shutdown(void)
     g_list_free(values);
 }
 
+void
+plugins_on_start(void)
+{
+    prof_add_shutdown_routine(_plugins_on_shutdown);
+    GList* values = g_hash_table_get_values(plugins);
+    GList* curr = values;
+    while (curr) {
+        ProfPlugin* plugin = curr->data;
+        plugin->on_start_func(plugin);
+        curr = g_list_next(curr);
+    }
+    g_list_free(values);
+}
+
 void
 plugins_on_connect(const char* const account_name, const char* const fulljid)
 {
@@ -476,18 +517,21 @@ plugins_on_disconnect(const char* const account_name, const char* const fulljid)
 char*
 plugins_pre_chat_message_display(const char* const barejid, const char* const resource, const char* message)
 {
+    GList* values = g_hash_table_get_values(plugins);
+    if (!values) {
+        return NULL;
+    }
+
     char* new_message = NULL;
     char* curr_message = strdup(message);
 
-    GList* values = g_hash_table_get_values(plugins);
     GList* curr = values;
     while (curr) {
         ProfPlugin* plugin = curr->data;
         new_message = plugin->pre_chat_message_display(plugin, barejid, resource, curr_message);
         if (new_message) {
             free(curr_message);
-            curr_message = strdup(new_message);
-            free(new_message);
+            curr_message = new_message;
         }
         curr = g_list_next(curr);
     }
@@ -512,21 +556,23 @@ plugins_post_chat_message_display(const char* const barejid, const char* const r
 char*
 plugins_pre_chat_message_send(const char* const barejid, const char* message)
 {
+    GList* values = g_hash_table_get_values(plugins);
+    if (!values) {
+        return NULL;
+    }
+
     char* new_message = NULL;
     char* curr_message = strdup(message);
 
-    GList* values = g_hash_table_get_values(plugins);
     GList* curr = values;
     while (curr) {
         ProfPlugin* plugin = curr->data;
         if (plugin->contains_hook(plugin, "prof_pre_chat_message_send")) {
             new_message = plugin->pre_chat_message_send(plugin, barejid, curr_message);
+            free(curr_message);
             if (new_message) {
-                free(curr_message);
-                curr_message = strdup(new_message);
-                free(new_message);
+                curr_message = new_message;
             } else {
-                free(curr_message);
                 g_list_free(values);
 
                 return NULL;
@@ -555,18 +601,21 @@ plugins_post_chat_message_send(const char* const barejid, const char* message)
 char*
 plugins_pre_room_message_display(const char* const barejid, const char* const nick, const char* message)
 {
+    GList* values = g_hash_table_get_values(plugins);
+    if (!values) {
+        return NULL;
+    }
+
     char* new_message = NULL;
     char* curr_message = strdup(message);
 
-    GList* values = g_hash_table_get_values(plugins);
     GList* curr = values;
     while (curr) {
         ProfPlugin* plugin = curr->data;
         new_message = plugin->pre_room_message_display(plugin, barejid, nick, curr_message);
         if (new_message) {
             free(curr_message);
-            curr_message = strdup(new_message);
-            free(new_message);
+            curr_message = new_message;
         }
         curr = g_list_next(curr);
     }
@@ -591,21 +640,23 @@ plugins_post_room_message_display(const char* const barejid, const char* const n
 char*
 plugins_pre_room_message_send(const char* const barejid, const char* message)
 {
+    GList* values = g_hash_table_get_values(plugins);
+    if (!values) {
+        return NULL;
+    }
+
     char* new_message = NULL;
     char* curr_message = strdup(message);
 
-    GList* values = g_hash_table_get_values(plugins);
     GList* curr = values;
     while (curr) {
         ProfPlugin* plugin = curr->data;
         if (plugin->contains_hook(plugin, "prof_pre_room_message_send")) {
             new_message = plugin->pre_room_message_send(plugin, barejid, curr_message);
+            free(curr_message);
             if (new_message) {
-                free(curr_message);
-                curr_message = strdup(new_message);
-                free(new_message);
+                curr_message = new_message;
             } else {
-                free(curr_message);
                 g_list_free(values);
 
                 return NULL;
@@ -657,19 +708,22 @@ plugins_on_room_history_message(const char* const barejid, const char* const nic
 char*
 plugins_pre_priv_message_display(const char* const fulljid, const char* message)
 {
+    GList* values = g_hash_table_get_values(plugins);
+    if (!values) {
+        return NULL;
+    }
+
     auto_jid Jid* jidp = jid_create(fulljid);
     char* new_message = NULL;
     char* curr_message = strdup(message);
 
-    GList* values = g_hash_table_get_values(plugins);
     GList* curr = values;
     while (curr) {
         ProfPlugin* plugin = curr->data;
         new_message = plugin->pre_priv_message_display(plugin, jidp->barejid, jidp->resourcepart, curr_message);
         if (new_message) {
             free(curr_message);
-            curr_message = strdup(new_message);
-            free(new_message);
+            curr_message = new_message;
         }
         curr = g_list_next(curr);
     }
@@ -695,22 +749,24 @@ plugins_post_priv_message_display(const char* const fulljid, const char* message
 char*
 plugins_pre_priv_message_send(const char* const fulljid, const char* const message)
 {
+    GList* values = g_hash_table_get_values(plugins);
+    if (!values) {
+        return NULL;
+    }
+
     auto_jid Jid* jidp = jid_create(fulljid);
     char* new_message = NULL;
     char* curr_message = strdup(message);
 
-    GList* values = g_hash_table_get_values(plugins);
     GList* curr = values;
     while (curr) {
         ProfPlugin* plugin = curr->data;
         if (plugin->contains_hook(plugin, "prof_pre_priv_message_send")) {
             new_message = plugin->pre_priv_message_send(plugin, jidp->barejid, jidp->resourcepart, curr_message);
+            free(curr_message);
             if (new_message) {
-                free(curr_message);
-                curr_message = strdup(new_message);
-                free(new_message);
+                curr_message = new_message;
             } else {
-                free(curr_message);
                 g_list_free(values);
 
                 return NULL;
@@ -742,18 +798,21 @@ plugins_post_priv_message_send(const char* const fulljid, const char* const mess
 char*
 plugins_on_message_stanza_send(const char* const text)
 {
+    GList* values = g_hash_table_get_values(plugins);
+    if (!values) {
+        return NULL;
+    }
+
     char* new_stanza = NULL;
     char* curr_stanza = strdup(text);
 
-    GList* values = g_hash_table_get_values(plugins);
     GList* curr = values;
     while (curr) {
         ProfPlugin* plugin = curr->data;
         new_stanza = plugin->on_message_stanza_send(plugin, curr_stanza);
         if (new_stanza) {
             free(curr_stanza);
-            curr_stanza = strdup(new_stanza);
-            free(new_stanza);
+            curr_stanza = new_stanza;
         }
         curr = g_list_next(curr);
     }
@@ -785,18 +844,21 @@ plugins_on_message_stanza_receive(const char* const text)
 char*
 plugins_on_presence_stanza_send(const char* const text)
 {
+    GList* values = g_hash_table_get_values(plugins);
+    if (!values) {
+        return NULL;
+    }
+
     char* new_stanza = NULL;
     char* curr_stanza = strdup(text);
 
-    GList* values = g_hash_table_get_values(plugins);
     GList* curr = values;
     while (curr) {
         ProfPlugin* plugin = curr->data;
         new_stanza = plugin->on_presence_stanza_send(plugin, curr_stanza);
         if (new_stanza) {
             free(curr_stanza);
-            curr_stanza = strdup(new_stanza);
-            free(new_stanza);
+            curr_stanza = new_stanza;
         }
         curr = g_list_next(curr);
     }
@@ -828,18 +890,21 @@ plugins_on_presence_stanza_receive(const char* const text)
 char*
 plugins_on_iq_stanza_send(const char* const text)
 {
+    GList* values = g_hash_table_get_values(plugins);
+    if (!values) {
+        return NULL;
+    }
+
     char* new_stanza = NULL;
     char* curr_stanza = strdup(text);
 
-    GList* values = g_hash_table_get_values(plugins);
     GList* curr = values;
     while (curr) {
         ProfPlugin* plugin = curr->data;
         new_stanza = plugin->on_iq_stanza_send(plugin, curr_stanza);
         if (new_stanza) {
             free(curr_stanza);
-            curr_stanza = strdup(new_stanza);
-            free(new_stanza);
+            curr_stanza = new_stanza;
         }
         curr = g_list_next(curr);
     }
@@ -925,40 +990,3 @@ plugins_get_disco_features(void)
 {
     return disco_get_features();
 }
-
-void
-plugins_shutdown(void)
-{
-    GList* values = g_hash_table_get_values(plugins);
-    GList* curr = values;
-
-    while (curr) {
-#ifdef HAVE_PYTHON
-        if (((ProfPlugin*)curr->data)->lang == LANG_PYTHON) {
-            python_plugin_destroy(curr->data);
-        }
-#endif
-#ifdef HAVE_C
-        if (((ProfPlugin*)curr->data)->lang == LANG_C) {
-            c_plugin_destroy(curr->data);
-        }
-#endif
-
-        curr = g_list_next(curr);
-    }
-    g_list_free(values);
-#ifdef HAVE_PYTHON
-    python_shutdown();
-#endif
-#ifdef HAVE_C
-    c_shutdown();
-#endif
-
-    autocompleters_destroy();
-    plugin_themes_close();
-    plugin_settings_close();
-    callbacks_close();
-    disco_close();
-    g_hash_table_destroy(plugins);
-    plugins = NULL;
-}
