@@ -2,7 +2,7 @@
  * editor.c
  * vim: expandtab:ts=4:sts=4:sw=4
  *
- * Copyright (C) 2022 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2022 - 2025 Michael Vetter <jubalh@iodoru.org>
  * Copyright (C) 2022 MarcoPolo PasTonMolo <marcopolopastonmolo@protonmail.com>
  * Copyright (C) 2022 Steffen Jaeckel <jaeckel-floss@eyet-services.de>
  *
@@ -56,7 +56,7 @@ get_message_from_editor(gchar* message, gchar** returned_message)
 
     auto_gchar gchar* filename = NULL;
     GError* glib_error = NULL;
-    auto_char char* jid = connection_get_barejid();
+    const char* jid = connection_get_barejid();
     if (jid) {
         filename = files_file_in_account_data_path(DIR_EDITOR, jid, "compose.md");
     } else {
