@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -67,8 +67,7 @@ gboolean
 _pgp_automatic_start(const char* const recipient)
 {
     gboolean result = FALSE;
-    char* account_name = session_get_account_name();
-    ProfAccount* account = accounts_get_account(account_name);
+    ProfAccount* account = accounts_get_account(session_get_account_name());
 
     if (g_list_find_custom(account->pgp_enabled, recipient, (GCompareFunc)g_strcmp0)) {
         result = TRUE;
@@ -82,8 +81,7 @@ gboolean
 _ox_automatic_start(const char* const recipient)
 {
     gboolean result = FALSE;
-    char* account_name = session_get_account_name();
-    ProfAccount* account = accounts_get_account(account_name);
+    ProfAccount* account = accounts_get_account(session_get_account_name());
 
     if (g_list_find_custom(account->ox_enabled, recipient, (GCompareFunc)g_strcmp0)) {
         result = TRUE;
@@ -295,19 +293,7 @@ chatwin_recipient_gone(ProfChatWin* chatwin)
 {
     assert(chatwin != NULL);
 
-    const char* display_usr = NULL;
-    PContact contact = roster_get_contact(chatwin->barejid);
-    if (contact) {
-        if (p_contact_name(contact)) {
-            display_usr = p_contact_name(contact);
-        } else {
-            display_usr = chatwin->barejid;
-        }
-    } else {
-        display_usr = chatwin->barejid;
-    }
-
-    win_println((ProfWin*)chatwin, THEME_GONE, "!", "<- %s has left the conversation.", display_usr);
+    win_println((ProfWin*)chatwin, THEME_GONE, "!", "<- %s has left the conversation.", roster_get_display_name(chatwin->barejid));
 }
 
 void
@@ -321,20 +307,16 @@ chatwin_incoming_msg(ProfChatWin* chatwin, ProfMessage* message, gboolean win_cr
     }
 
     char* old_plain = message->plain;
+    auto_char char* new_plain = plugins_pre_chat_message_display(message->from_jid->barejid, message->from_jid->resourcepart, message->plain);
+    if (new_plain)
+        message->plain = new_plain;
 
-    message->plain = plugins_pre_chat_message_display(message->from_jid->barejid, message->from_jid->resourcepart, message->plain);
     gboolean show_message = true;
 
     ProfWin* window = (ProfWin*)chatwin;
     int num = wins_get_num(window);
 
-    auto_gchar gchar* display_name;
-    auto_char char* mybarejid = connection_get_barejid();
-    if (g_strcmp0(mybarejid, message->from_jid->barejid) == 0) {
-        display_name = strdup("me");
-    } else {
-        display_name = roster_get_msg_display_name(message->from_jid->barejid, message->from_jid->resourcepart);
-    }
+    auto_gchar gchar* display_name = get_display_name(message, NULL);
 
 #ifdef HAVE_LIBGPGME
     if (prefs_get_boolean(PREF_PGP_PUBKEY_AUTOIMPORT)) {
@@ -417,12 +399,11 @@ chatwin_incoming_msg(ProfChatWin* chatwin, ProfMessage* message, gboolean win_cr
 
     plugins_post_chat_message_display(message->from_jid->barejid, message->from_jid->resourcepart, message->plain);
 
-    free(message->plain);
     message->plain = old_plain;
 }
 
 void
-chatwin_outgoing_msg(ProfChatWin* chatwin, const char* const message, char* id, prof_enc_t enc_mode,
+chatwin_outgoing_msg(ProfChatWin* chatwin, const char* const message, const char* id, prof_enc_t enc_mode,
                      gboolean request_receipt, const char* const replace_id)
 {
     assert(chatwin != NULL);
@@ -430,30 +411,21 @@ chatwin_outgoing_msg(ProfChatWin* chatwin, const char* const message, char* id,
     ProfWin* window = (ProfWin*)chatwin;
     wins_add_quotes_ac(window, message, FALSE);
 
-    auto_char char* enc_char;
-    if (chatwin->outgoing_char) {
-        enc_char = chatwin->outgoing_char;
-    } else if (enc_mode == PROF_MSG_ENC_OTR) {
-        enc_char = prefs_get_otr_char();
-    } else if (enc_mode == PROF_MSG_ENC_PGP) {
-        enc_char = prefs_get_pgp_char();
-    } else if (enc_mode == PROF_MSG_ENC_OMEMO) {
-        enc_char = prefs_get_omemo_char();
-    } else if (enc_mode == PROF_MSG_ENC_OX) {
-        enc_char = prefs_get_ox_char();
-    } else {
-        enc_char = strdup("-");
-    }
+    auto_char char* enc_char = get_enc_char(enc_mode, chatwin->outgoing_char);
+
+    const Jid* myjid = connection_get_jid();
+    auto_char char* plugin_message = plugins_pre_chat_message_display(myjid->barejid, myjid->resourcepart, message);
+    const char* display_message = plugin_message ?: message;
 
     if (request_receipt && id) {
-        win_print_outgoing_with_receipt((ProfWin*)chatwin, enc_char, "me", message, id, replace_id);
+        win_print_outgoing_with_receipt((ProfWin*)chatwin, enc_char, "me", display_message, id, replace_id);
     } else {
-        win_print_outgoing((ProfWin*)chatwin, enc_char, id, replace_id, message);
+        win_print_outgoing((ProfWin*)chatwin, enc_char, id, replace_id, display_message);
     }
 
-    // save last id and message for LMC
-    if (id) {
-        _chatwin_set_last_message(chatwin, id, message);
+    // save last id and message for LMC in case if it's not LMC message
+    if (id && !replace_id) {
+        _chatwin_set_last_message(chatwin, id, display_message);
     }
 }
 
@@ -462,11 +434,13 @@ chatwin_outgoing_carbon(ProfChatWin* chatwin, ProfMessage* message)
 {
     assert(chatwin != NULL);
 
-    auto_char char* enc_char;
+    auto_char char* enc_char = NULL;
     if (message->enc == PROF_MSG_ENC_PGP) {
         enc_char = prefs_get_pgp_char();
     } else if (message->enc == PROF_MSG_ENC_OMEMO) {
         enc_char = prefs_get_omemo_char();
+    } else if (message->enc == PROF_MSG_ENC_OX) {
+        enc_char = prefs_get_ox_char();
     } else {
         enc_char = strdup("-");
     }
@@ -501,7 +475,7 @@ chatwin_contact_offline(ProfChatWin* chatwin, char* resource, char* status)
     win_show_status_string((ProfWin*)chatwin, display_str, "offline", status, NULL, "--", "offline");
 }
 
-char*
+gchar*
 chatwin_get_string(ProfChatWin* chatwin)
 {
     assert(chatwin != NULL);
@@ -592,12 +566,12 @@ _chatwin_history(ProfChatWin* chatwin, const char* const contact_barejid)
 
         while (curr) {
             ProfMessage* msg = curr->data;
-            char* msg_plain = msg->plain;
-            msg->plain = plugins_pre_chat_message_display(msg->from_jid->barejid, msg->from_jid->resourcepart, msg->plain);
-            // This is dirty workaround for memory leak. We reassign msg->plain above so have to free previous object
-            // TODO: Make a better solution, for example, pass msg object to the function and it will replace msg->plain properly if needed.
-            free(msg_plain);
+            char* old_plain = msg->plain;
+            auto_char char* plugin_msg = plugins_pre_chat_message_display(msg->from_jid->barejid, msg->from_jid->resourcepart, msg->plain);
+            if (plugin_msg)
+                msg->plain = plugin_msg;
             win_print_history((ProfWin*)chatwin, msg);
+            msg->plain = old_plain;
             curr = g_slist_next(curr);
         }
         chatwin->history_shown = TRUE;
@@ -622,16 +596,16 @@ chatwin_db_history(ProfChatWin* chatwin, const char* start_time, char* end_time,
 
     while (curr) {
         ProfMessage* msg = curr->data;
-        char* msg_plain = msg->plain;
-        msg->plain = plugins_pre_chat_message_display(msg->from_jid->barejid, msg->from_jid->resourcepart, msg->plain);
-        // This is dirty workaround for memory leak. We reassign msg->plain above so have to free previous object
-        // TODO: Make a better solution, for example, pass msg object to the function and it will replace msg->plain properly if needed.
-        free(msg_plain);
+        char* old_plain = msg->plain;
+        auto_char char* plugin_msg = plugins_pre_chat_message_display(msg->from_jid->barejid, msg->from_jid->resourcepart, msg->plain);
+        if (plugin_msg)
+            msg->plain = plugin_msg;
         if (flip) {
             win_print_old_history((ProfWin*)chatwin, msg);
         } else {
             win_print_history((ProfWin*)chatwin, msg);
         }
+        msg->plain = old_plain;
         curr = g_slist_next(curr);
     }
 
