@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -196,7 +196,7 @@ mucwin_room_disco_info(ProfMucWin* mucwin, GSList* identities, GSList* features)
             win_println(window, THEME_DEFAULT, "!", "Identities:");
         }
         while (identities) {
-            DiscoIdentity* identity = identities->data; // anme trpe, cat
+            DiscoIdentity* identity = identities->data; // name, type, category
             GString* identity_str = g_string_new("  ");
             if (identity->name) {
                 identity_str = g_string_append(identity_str, identity->name);
@@ -368,7 +368,7 @@ mucwin_history(ProfMucWin* mucwin, const ProfMessage* const message)
     assert(mucwin != NULL);
 
     char* nick = message->from_jid->resourcepart;
-    char* mynick = muc_nick(mucwin->roomjid);
+    const char* const mynick = muc_nick(mucwin->roomjid);
     GSList* mentions = get_mentions(prefs_get_boolean(PREF_NOTIFY_MENTION_WHOLE_WORD), prefs_get_boolean(PREF_NOTIFY_MENTION_CASE_SENSITIVE), message->plain, mynick);
     GList* triggers = prefs_message_get_triggers(message->plain);
 
@@ -391,7 +391,7 @@ _mucwin_print_mention(ProfWin* window, const char* const message, const char* co
     while (curr) {
         pos = GPOINTER_TO_INT(curr->data);
 
-        char* before_str = g_utf8_substring(message, last_pos, pos);
+        auto_gchar gchar* before_str = g_utf8_substring(message, last_pos, pos);
 
         if (last_pos == 0 && strncmp(before_str, "/me ", 4) == 0) {
             win_print_them(window, THEME_ROOMMENTION, ch, flags, "");
@@ -404,11 +404,9 @@ _mucwin_print_mention(ProfWin* window, const char* const message, const char* co
             }
             win_append_highlight(window, THEME_ROOMMENTION, "%s", before_str);
         }
-        g_free(before_str);
 
-        char* mynick_str = g_utf8_substring(message, pos, pos + mynick_len);
+        auto_gchar gchar* mynick_str = g_utf8_substring(message, pos, pos + mynick_len);
         win_append_highlight(window, THEME_ROOMMENTION_TERM, "%s", mynick_str);
-        g_free(mynick_str);
 
         last_pos = pos + mynick_len;
 
@@ -418,7 +416,7 @@ _mucwin_print_mention(ProfWin* window, const char* const message, const char* co
     glong message_len = g_utf8_strlen(message, -1);
     if (last_pos < message_len) {
         // get tail without allocating a new string
-        char* rest = g_utf8_offset_to_pointer(message, last_pos);
+        gchar* rest = g_utf8_offset_to_pointer(message, last_pos);
         win_appendln_highlight(window, THEME_ROOMMENTION, "%s", rest);
     } else {
         win_appendln_highlight(window, THEME_ROOMMENTION, "");
@@ -449,15 +447,15 @@ _mucwin_print_triggers(ProfWin* window, const char* const message, GList* trigge
         curr = g_list_next(curr);
     }
 
-    char* message_lower = g_utf8_strdown(message, -1);
+    auto_gchar gchar* message_lower = g_utf8_strdown(message, -1);
 
     // find earliest trigger in message
     int first_trigger_pos = -1;
     int first_trigger_len = -1;
     curr = weighted_triggers;
     while (curr) {
-        char* trigger_lower = g_utf8_strdown(curr->data, -1);
-        char* trigger_ptr = g_strstr_len(message_lower, -1, trigger_lower);
+        auto_gchar gchar* trigger_lower = g_utf8_strdown(curr->data, -1);
+        gchar* trigger_ptr = g_strstr_len(message_lower, -1, trigger_lower);
 
         // not found, try next
         if (trigger_ptr == NULL) {
@@ -472,11 +470,9 @@ _mucwin_print_triggers(ProfWin* window, const char* const message, GList* trigge
             first_trigger_len = strlen(trigger_lower);
         }
 
-        g_free(trigger_lower);
         curr = g_list_next(curr);
     }
 
-    g_free(message_lower);
     g_list_free(weighted_triggers);
 
     // no triggers found
@@ -516,33 +512,19 @@ mucwin_outgoing_msg(ProfMucWin* mucwin, const char* const message, const char* c
     assert(mucwin != NULL);
 
     ProfWin* window = (ProfWin*)mucwin;
-    char* mynick = muc_nick(mucwin->roomjid);
+    const char* const mynick = muc_nick(mucwin->roomjid);
 
     // displayed message char
-    char* ch;
-    if (mucwin->message_char) {
-        ch = strdup(mucwin->message_char);
-    } else if (enc_mode == PROF_MSG_ENC_OTR) {
-        ch = prefs_get_otr_char();
-    } else if (enc_mode == PROF_MSG_ENC_PGP) {
-        ch = prefs_get_pgp_char();
-    } else if (enc_mode == PROF_MSG_ENC_OMEMO) {
-        ch = prefs_get_omemo_char();
-    } else if (enc_mode == PROF_MSG_ENC_OX) {
-        ch = prefs_get_omemo_char();
-    } else {
-        ch = strdup("-");
-    }
+    auto_char char* ch = get_enc_char(enc_mode, mucwin->message_char);
 
     win_print_outgoing_muc_msg(window, ch, mynick, id, replace_id, message);
-    free(ch);
 
     // save last id and message for LMC
     if (id) {
         _mucwin_set_last_message(mucwin, id, message);
     }
 
-    wins_add_quotes_ac(window, message);
+    wins_add_quotes_ac(window, message, FALSE);
 }
 
 void
@@ -566,24 +548,13 @@ mucwin_incoming_msg(ProfMucWin* mucwin, const ProfMessage* const message, GSList
     }
 
     ProfWin* window = (ProfWin*)mucwin;
-    char* mynick = muc_nick(mucwin->roomjid);
+    const char* const mynick = muc_nick(mucwin->roomjid);
 
-    char* ch;
-    if (mucwin->message_char) {
-        ch = strdup(mucwin->message_char);
-    } else if (message->enc == PROF_MSG_ENC_OTR) {
-        ch = prefs_get_otr_char();
-    } else if (message->enc == PROF_MSG_ENC_PGP) {
-        ch = prefs_get_pgp_char();
-    } else if (message->enc == PROF_MSG_ENC_OMEMO) {
-        ch = prefs_get_omemo_char();
-    } else {
-        ch = strdup("-");
-    }
+    auto_char char* ch = get_enc_char(message->enc, mucwin->message_char);
 
     win_insert_last_read_position_marker((ProfWin*)mucwin, mucwin->roomjid);
-    wins_add_urls_ac(window, message);
-    wins_add_quotes_ac(window, message->plain);
+    wins_add_urls_ac(window, message, FALSE);
+    wins_add_quotes_ac(window, message->plain, FALSE);
 
     if (g_slist_length(mentions) > 0) {
         _mucwin_print_mention(window, message->plain, message->from_jid->resourcepart, mynick, mentions, ch, flags);
@@ -593,8 +564,6 @@ mucwin_incoming_msg(ProfMucWin* mucwin, const ProfMessage* const message, GSList
     } else {
         win_println_incoming_muc_msg(window, ch, flags, message);
     }
-
-    free(ch);
 }
 
 void
@@ -938,22 +907,16 @@ mucwin_hide_occupants(ProfMucWin* mucwin)
     }
 }
 
-char*
+gchar*
 mucwin_get_string(ProfMucWin* mucwin)
 {
     assert(mucwin != NULL);
 
-    GString* res = g_string_new("Room ");
-    g_string_append(res, mucwin->roomjid);
-
     if (mucwin->unread > 0) {
-        g_string_append_printf(res, ", %d unread", mucwin->unread);
+        return g_strdup_printf("Room %s, %d unread", mucwin->roomjid, mucwin->unread);
+    } else {
+        return g_strdup_printf("Room %s", mucwin->roomjid);
     }
-
-    char* resstr = res->str;
-    g_string_free(res, FALSE);
-
-    return resstr;
 }
 
 void
@@ -1001,3 +964,48 @@ _mucwin_set_last_message(ProfMucWin* mucwin, const char* const id, const char* c
     free(mucwin->last_msg_id);
     mucwin->last_msg_id = strdup(id);
 }
+
+gchar*
+mucwin_generate_title(const gchar* const muc_jid, const preference_t pref)
+{
+    auto_gchar gchar* pref_val = prefs_get_string(pref);
+    if (g_strcmp0(pref_val, "name") == 0) {
+        const ProfMucWin* mucwin = wins_get_muc(muc_jid);
+        if (mucwin && mucwin->room_name) {
+            return g_strdup(mucwin->room_name);
+        }
+    } else if (g_strcmp0(pref_val, "bookmark") == 0) {
+        const Bookmark* bookmark = bookmark_get_by_jid(muc_jid);
+        if (bookmark && bookmark->name) {
+            return g_strdup(bookmark->name);
+        }
+    }
+
+    auto_gchar gchar* roster_room_by = prefs_get_string(PREF_ROSTER_ROOMS_BY);
+    if (g_strcmp0(pref_val, "localpart") == 0 || (g_strcmp0(roster_room_by, "service") == 0 && pref == PREF_ROSTER_ROOMS_TITLE)) {
+        auto_jid Jid* jid = jid_create(muc_jid);
+        if (jid && jid->localpart) {
+            return g_strdup(jid->localpart);
+        }
+    }
+
+    return g_strdup(muc_jid);
+}
+
+gboolean
+mucwin_set_room_name(const gchar* const muc_jid, const gchar* const new_room_name)
+{
+    ProfMucWin* mucwin = wins_get_muc(muc_jid);
+    if (!mucwin) {
+        log_error("No window found with this JID: '%s'", muc_jid);
+        return FALSE;
+    }
+
+    free(mucwin->room_name);
+    if (new_room_name) {
+        mucwin->room_name = strdup(new_room_name);
+    } else {
+        mucwin->room_name = NULL;
+    }
+    return TRUE;
+}
