@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -307,7 +307,7 @@ _rosterwin_unsubscribed_item(ProfLayoutSplit* layout, ProfChatWin* chatwin)
             indent--;
         }
     }
-    auto_char char* ch = prefs_get_roster_contact_char();
+    auto_gchar gchar* ch = prefs_get_roster_contact_char();
     if (ch) {
         g_string_append_printf(msg, "%s", ch);
     }
@@ -368,7 +368,7 @@ _rosterwin_contact(ProfLayoutSplit* layout, PContact contact)
             indent--;
         }
     }
-    auto_char char* ch = prefs_get_roster_contact_char();
+    auto_gchar gchar* ch = prefs_get_roster_contact_char();
     if (ch) {
         g_string_append_printf(msg, "%s", ch);
     }
@@ -517,7 +517,7 @@ _rosterwin_resources(ProfLayoutSplit* layout, PContact contact, int current_inde
 
             wattron(layout->subwin, theme_attrs(resource_presence_colour));
             GString* msg = g_string_new("");
-            auto_char char* ch = prefs_get_roster_resource_char();
+            auto_gchar gchar* ch = prefs_get_roster_resource_char();
             if (ch) {
                 g_string_append_printf(msg, "%s", ch);
             } else {
@@ -735,7 +735,7 @@ _rosterwin_room(ProfLayoutSplit* layout, ProfMucWin* mucwin)
             indent--;
         }
     }
-    auto_char char* ch = prefs_get_roster_room_char();
+    auto_gchar gchar* ch = prefs_get_roster_room_char();
     if (ch) {
         g_string_append_printf(msg, "%s", ch);
     }
@@ -745,35 +745,8 @@ _rosterwin_room(ProfLayoutSplit* layout, ProfMucWin* mucwin)
         g_string_append_printf(msg, "(%d) ", mucwin->unread);
     }
 
-    auto_gchar gchar* use_as_name = prefs_get_string(PREF_ROSTER_ROOMS_USE_AS_NAME);
-    auto_gchar gchar* roombypref = prefs_get_string(PREF_ROSTER_ROOMS_BY);
-
-    if (g_strcmp0(roombypref, "service") == 0) {
-        if (mucwin->room_name == NULL || (g_strcmp0(use_as_name, "jid") == 0)) {
-            auto_jid Jid* jidp = jid_create(mucwin->roomjid);
-            g_string_append(msg, jidp->localpart);
-        } else {
-            g_string_append(msg, mucwin->room_name);
-        }
-    } else {
-        gboolean show_server = prefs_get_boolean(PREF_ROSTER_ROOMS_SERVER);
-
-        if (show_server) {
-            if (mucwin->room_name == NULL || (g_strcmp0(use_as_name, "jid") == 0)) {
-                g_string_append(msg, mucwin->roomjid);
-            } else {
-                g_string_append(msg, mucwin->room_name);
-            }
-        } else {
-            auto_jid Jid* jidp = jid_create(mucwin->roomjid);
-
-            if (mucwin->room_name == NULL || (g_strcmp0(use_as_name, "jid") == 0)) {
-                g_string_append(msg, jidp->localpart);
-            } else {
-                g_string_append(msg, mucwin->room_name);
-            }
-        }
-    }
+    auto_gchar gchar* mucwin_title = mucwin_generate_title(mucwin->roomjid, PREF_ROSTER_ROOMS_TITLE);
+    g_string_append(msg, mucwin_title);
 
     if ((g_strcmp0(unreadpos, "after") == 0) && mucwin->unread > 0) {
         g_string_append_printf(msg, " (%d)", mucwin->unread);
@@ -828,7 +801,7 @@ _rosterwin_room(ProfLayoutSplit* layout, ProfMucWin* mucwin)
                 g_string_append_printf(privmsg, "(%d) ", privwin->unread);
             }
 
-            auto_char char* ch2 = prefs_get_roster_room_private_char();
+            auto_gchar gchar* ch2 = prefs_get_roster_room_private_char();
             if (ch2) {
                 g_string_append_printf(privmsg, "%s", ch2);
             }
@@ -919,7 +892,7 @@ _rosterwin_private_chats(ProfLayoutSplit* layout, GList* orphaned_privchats)
                 g_string_append_printf(privmsg, "(%d) ", privwin->unread);
             }
 
-            auto_char char* ch = prefs_get_roster_private_char();
+            auto_gchar gchar* ch = prefs_get_roster_private_char();
             if (ch) {
                 g_string_append_printf(privmsg, "%s", ch);
             }
@@ -998,7 +971,7 @@ _rosterwin_unsubscribed_header(ProfLayoutSplit* layout, GList* wins)
     win_sub_newline_lazy(layout->subwin);
 
     GString* header = g_string_new(" ");
-    auto_char char* ch = prefs_get_roster_header_char();
+    auto_gchar gchar* ch = prefs_get_roster_header_char();
     if (ch) {
         g_string_append_printf(header, "%s", ch);
     }
@@ -1043,7 +1016,7 @@ _rosterwin_contacts_header(ProfLayoutSplit* layout, const char* const title, GSL
     win_sub_newline_lazy(layout->subwin);
 
     GString* header = g_string_new(" ");
-    auto_char char* ch = prefs_get_roster_header_char();
+    auto_gchar gchar* ch = prefs_get_roster_header_char();
     if (ch) {
         g_string_append_printf(header, "%s", ch);
     }
@@ -1093,7 +1066,7 @@ _rosterwin_rooms_header(ProfLayoutSplit* layout, GList* rooms, char* title)
 {
     win_sub_newline_lazy(layout->subwin);
     GString* header = g_string_new(" ");
-    auto_char char* ch = prefs_get_roster_header_char();
+    auto_gchar gchar* ch = prefs_get_roster_header_char();
     if (ch) {
         g_string_append_printf(header, "%s", ch);
     }
@@ -1154,7 +1127,7 @@ _rosterwin_private_header(ProfLayoutSplit* layout, GList* privs)
     win_sub_newline_lazy(layout->subwin);
 
     GString* title_str = g_string_new(" ");
-    auto_char char* ch = prefs_get_roster_header_char();
+    auto_gchar gchar* ch = prefs_get_roster_header_char();
     if (ch) {
         g_string_append_printf(title_str, "%s", ch);
     }
