@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -85,6 +85,13 @@ create_title_bar(void)
     inp_put_back();
 }
 
+void
+free_title_bar(void)
+{
+    delwin(win);
+    win = NULL;
+}
+
 void
 title_bar_update_virtual(void)
 {
@@ -195,7 +202,7 @@ _title_bar_draw(void)
         waddch(win, ' ');
     }
 
-    auto_char char* title = win_get_title(current);
+    auto_gchar gchar* title = win_get_title(current);
 
     mvwprintw(win, 0, 0, " %s", title);
     pos = strlen(title) + 1;
