@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2018 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2018 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -68,6 +68,7 @@ typedef struct prof_conn_t
     char* presence_message;
     int priority;
     char* domain;
+    Jid* jid;
     GHashTable* available_resources;
     GHashTable* features_by_jid;
     GHashTable* requested_features;
@@ -135,6 +136,7 @@ connection_init(void)
     conn.conn_last_event = XMPP_CONN_DISCONNECT;
     conn.presence_message = NULL;
     conn.domain = NULL;
+    conn.jid = NULL;
     conn.features_by_jid = NULL;
     conn.available_resources = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)resource_destroy);
     conn.requested_features = g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL);
@@ -165,6 +167,12 @@ void
 connection_shutdown(void)
 {
     connection_clear_data();
+    g_hash_table_destroy(conn.requested_features);
+    g_hash_table_destroy(conn.available_resources);
+    if (conn.sm_state) {
+        xmpp_free_sm_state(conn.sm_state);
+        conn.sm_state = NULL;
+    }
     if (conn.xmpp_conn) {
         xmpp_conn_release(conn.xmpp_conn);
         conn.xmpp_conn = NULL;
@@ -190,7 +198,6 @@ _conn_apply_settings(const char* const jid, const char* const passwd, const char
 
     _compute_identifier(jidp->barejid);
 
-    xmpp_ctx_set_verbosity(conn.xmpp_ctx, 0);
     xmpp_conn_set_jid(conn.xmpp_conn, jid);
     if (passwd)
         xmpp_conn_set_pass(conn.xmpp_conn, passwd);
@@ -561,8 +568,11 @@ connection_disconnect(void)
         }
     }
 
-    free(prof_identifier);
+    g_free(prof_identifier);
     prof_identifier = NULL;
+
+    jid_destroy(conn.jid);
+    conn.jid = NULL;
 }
 
 void
@@ -644,7 +654,7 @@ connection_supports(const char* const feature)
     return ret;
 }
 
-char*
+const char*
 connection_jid_for_feature(const char* const feature)
 {
     if (conn.features_by_jid == NULL) {
@@ -725,31 +735,46 @@ connection_get_fulljid(void)
     }
 }
 
-char*
-connection_get_barejid(void)
+const Jid*
+connection_get_jid(void)
 {
-    const char* jid = connection_get_fulljid();
-    if (!jid)
-        return NULL;
-
-    auto_jid Jid* jidp = jid_create(jid);
-    char* result = strdup(jidp->barejid);
-
-    return result;
+    static const char* fulljid;
+    const char* cur_fulljid = connection_get_fulljid();
+    if (!conn.jid || cur_fulljid != fulljid) {
+        fulljid = cur_fulljid;
+        if (conn.jid)
+            jid_destroy(conn.jid);
+        conn.jid = jid_create(fulljid);
+        if (!conn.jid) {
+            log_error("Could not create connection-wide JID object from \"%s\"", STR_MAYBE_NULL(fulljid));
+            /* In case we failed to create the jid or we're not yet
+             * connected (or whatever else could fail) return the
+             * pointer to a zero-initialized static object, so
+             * de-referencing that pointer won't fail.
+             */
+            static const Jid jid = { 0 };
+            return &jid;
+        }
+    }
+    return conn.jid;
 }
 
-char*
+const char*
+connection_get_barejid(void)
+{
+    return connection_get_jid()->barejid;
+}
+
+gboolean
+equals_our_barejid(const char* cmp)
+{
+    return g_strcmp0(connection_get_jid()->barejid, cmp) == 0;
+}
+
+const char*
 connection_get_user(void)
 {
-    const char* jid = connection_get_fulljid();
-    if (!jid)
-        return NULL;
-    char* result = strdup(jid);
-
-    char* split = strchr(result, '@');
-    *split = '\0';
-
-    return result;
+    return connection_get_jid()->localpart;
 }
 
 void
@@ -815,18 +840,16 @@ connection_create_stanza_id(void)
                                                        (guchar*)prof_identifier, strlen(prof_identifier),
                                                        rndid, strlen(rndid));
 
-    char* ret = g_strdup_printf("%s%s", rndid, hmac);
-
-    return ret;
+    return g_strdup_printf("%s%s", rndid, hmac);
 }
 
-char*
+const char*
 connection_get_domain(void)
 {
     return conn.domain;
 }
 
-char*
+const char*
 connection_get_presence_msg(void)
 {
     return conn.presence_message;
@@ -954,9 +977,8 @@ _connection_handler(xmpp_conn_t* const xmpp_conn, const xmpp_conn_event_t status
         log_debug("Connection handler: XMPP_CONN_CONNECT");
         conn.conn_status = JABBER_CONNECTED;
 
-        Jid* my_jid = jid_create(xmpp_conn_get_jid(conn.xmpp_conn));
-        conn.domain = strdup(my_jid->domainpart);
-        jid_destroy(my_jid);
+        connection_get_jid();
+        conn.domain = strdup(conn.jid->domainpart);
 
         connection_clear_data();
         conn.features_by_jid = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)g_hash_table_destroy);
@@ -980,10 +1002,10 @@ _connection_handler(xmpp_conn_t* const xmpp_conn, const xmpp_conn_event_t status
         log_debug("Connection handler: XMPP_CONN_RAW_CONNECT");
         conn.conn_status = JABBER_RAW_CONNECTED;
 
-        Jid* my_raw_jid = jid_create(xmpp_conn_get_jid(conn.xmpp_conn));
-        log_debug("jid: %s", xmpp_conn_get_jid(conn.xmpp_conn));
-        conn.domain = strdup(my_raw_jid->domainpart);
-        jid_destroy(my_raw_jid);
+        connection_get_jid();
+        conn.jid = jid_create(xmpp_conn_get_jid(conn.xmpp_conn));
+        log_debug("jid: %s", conn.jid->str);
+        conn.domain = strdup(conn.jid->domainpart);
 
         connection_clear_data();
         conn.features_by_jid = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)g_hash_table_destroy);
@@ -998,22 +1020,25 @@ _connection_handler(xmpp_conn_t* const xmpp_conn, const xmpp_conn_event_t status
         log_debug("Connection handler: XMPP_CONN_DISCONNECT");
 
         // lost connection for unknown reason
-        if (conn.conn_status == JABBER_CONNECTED) {
+        if (conn.conn_status == JABBER_CONNECTED || conn.conn_status == JABBER_DISCONNECTING) {
             if (prefs_get_boolean(PREF_STROPHE_SM_ENABLED)) {
                 int send_queue_len = xmpp_conn_send_queue_len(conn.xmpp_conn);
-                log_debug("Connection handler: Lost connection for unknown reason");
+                log_debug("Connection handler: Lost connection for unknown reason, %d messages in send queue", send_queue_len);
                 conn.sm_state = xmpp_conn_get_sm_state(conn.xmpp_conn);
                 if (send_queue_len > 0 && prefs_get_boolean(PREF_STROPHE_SM_RESEND)) {
                     conn.queued_messages = calloc(send_queue_len + 1, sizeof(*conn.queued_messages));
                     for (int n = 0; n < send_queue_len && conn.queued_messages[n]; ++n) {
                         conn.queued_messages[n] = xmpp_conn_send_queue_drop_element(conn.xmpp_conn, XMPP_QUEUE_OLDEST);
                     }
+                } else if (send_queue_len > 0) {
+                    log_debug("Connection handler: dropping those messages since SM RESEND is disabled");
                 }
             }
-            session_lost_connection();
+            if (conn.conn_status == JABBER_CONNECTED)
+                session_lost_connection();
 
             // login attempt failed
-        } else if (conn.conn_status != JABBER_DISCONNECTING) {
+        } else {
             gchar* host;
             int port;
             if (stream_error && stream_error->stanza && _get_other_host(stream_error->stanza, &host, &port)) {
@@ -1102,31 +1127,18 @@ _xmpp_file_logger(void* const userdata, const xmpp_log_level_t xmpp_level, const
 static void
 _random_bytes_init(void)
 {
-    auto_gchar gchar* rndbytes_loc = files_get_data_path(FILE_PROFANITY_IDENTIFIER);
-    GKeyFile* rndbytes;
+    prof_keyfile_t keyfile;
+    load_data_keyfile(&keyfile, FILE_PROFANITY_IDENTIFIER);
 
-    if (g_file_test(rndbytes_loc, G_FILE_TEST_EXISTS)) {
-        g_chmod(rndbytes_loc, S_IRUSR | S_IWUSR);
-    }
-
-    rndbytes = g_key_file_new();
-    g_key_file_load_from_file(rndbytes, rndbytes_loc, G_KEY_FILE_KEEP_COMMENTS, NULL);
-
-    if (g_key_file_has_group(rndbytes, "identifier")) {
-        profanity_instance_id = g_key_file_get_string(rndbytes, "identifier", "random_bytes", NULL);
+    if (g_key_file_has_group(keyfile.keyfile, "identifier")) {
+        profanity_instance_id = g_key_file_get_string(keyfile.keyfile, "identifier", "random_bytes", NULL);
     } else {
         profanity_instance_id = get_random_string(10);
-        g_key_file_set_string(rndbytes, "identifier", "random_bytes", profanity_instance_id);
-
-        gsize g_data_size;
-        auto_gchar gchar* g_accounts_data = g_key_file_to_data(rndbytes, &g_data_size, NULL);
-
-        auto_gchar gchar* base = g_path_get_dirname(rndbytes_loc);
-        auto_gchar gchar* true_loc = get_file_or_linked(rndbytes_loc, base);
-        g_file_set_contents(true_loc, g_accounts_data, g_data_size, NULL);
+        g_key_file_set_string(keyfile.keyfile, "identifier", "random_bytes", profanity_instance_id);
+        save_keyfile(&keyfile);
     }
 
-    g_key_file_free(rndbytes);
+    free_keyfile(&keyfile);
 }
 
 static void
@@ -1139,7 +1151,7 @@ static void
 _compute_identifier(const char* barejid)
 {
     // in case of reconnect (lost connection)
-    free(prof_identifier);
+    g_free(prof_identifier);
 
     prof_identifier = g_compute_hmac_for_string(G_CHECKSUM_SHA256,
                                                 (guchar*)profanity_instance_id, strlen(profanity_instance_id),
