@@ -56,7 +56,7 @@ void connection_set_disco_items(GSList* items);
 
 xmpp_conn_t* connection_get_conn(void);
 xmpp_ctx_t* connection_get_ctx(void);
-char* connection_get_domain(void);
+const char* connection_get_domain(void);
 void connection_request_features(void);
 void connection_features_received(const char* const jid);
 GHashTable* connection_get_features(const char* const jid);
