@@ -63,7 +63,7 @@ gboolean jid_is_valid_room_form(Jid* jid);
 char* create_fulljid(const char* const barejid, const char* const resource);
 char* get_nick_from_full_jid(const char* const full_room_jid);
 
-char* jid_fulljid_or_barejid(Jid* jid);
+const char* jid_fulljid_or_barejid(Jid* jid);
 gchar* jid_random_resource(void);
 
 #endif
