@@ -91,28 +91,29 @@ static Occupant* _muc_occupant_new(const char* const nick, const char* const jid
                                    muc_affiliation_t affiliation, resource_presence_t presence, const char* const status);
 static void _occupant_free(Occupant* occupant);
 
+static void
+_muc_close(void)
+{
+    g_hash_table_destroy(invite_passwords);
+    invite_passwords = NULL;
+    g_hash_table_destroy(rooms);
+    rooms = NULL;
+    autocomplete_free(confservers_ac);
+    confservers_ac = NULL;
+    autocomplete_free(invite_ac);
+    invite_ac = NULL;
+}
+
 void
 muc_init(void)
 {
+    prof_add_shutdown_routine(_muc_close);
     invite_ac = autocomplete_new();
     confservers_ac = autocomplete_new();
     rooms = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify)_free_room);
     invite_passwords = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 }
 
-void
-muc_close(void)
-{
-    autocomplete_free(invite_ac);
-    autocomplete_free(confservers_ac);
-    g_hash_table_destroy(rooms);
-    g_hash_table_destroy(invite_passwords);
-    rooms = NULL;
-    invite_passwords = NULL;
-    invite_ac = NULL;
-    confservers_ac = NULL;
-}
-
 void
 muc_confserver_add(const char* const server)
 {
@@ -426,7 +427,7 @@ muc_rooms(void)
  * Return current users nickname for the specified room
  * The nickname is owned by the chat room and should not be modified or freed
  */
-char*
+const char* const
 muc_nick(const char* const room)
 {
     ChatRoom* chat_room = g_hash_table_lookup(rooms, room);
@@ -882,8 +883,7 @@ muc_members_add(const char* const room, const char* const jid)
         if (g_hash_table_insert(chat_room->members, strdup(jid), NULL)) {
 #ifdef HAVE_OMEMO
             if (chat_room->anonymity_type == MUC_ANONYMITY_TYPE_NONANONYMOUS) {
-                auto_char char* our_barejid = connection_get_barejid();
-                if (strcmp(jid, our_barejid) != 0) {
+                if (!equals_our_barejid(jid)) {
                     omemo_start_session(jid);
                 }
             }
