@@ -21,8 +21,7 @@
  */
 
 #include <glib.h>
-#include <setjmp.h>
-#include <cmocka.h>
+#include "prof_cmocka.h"
 
 #include <xmpp/xmpp.h>
 
