@@ -2,10 +2,7 @@
 #include <libotr/message.h>
 #include <glib.h>
 
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
+#include "prof_cmocka.h"
 
 #include "config/account.h"
 
