@@ -1,8 +1,5 @@
-#include <stdarg.h>
 #include <string.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
+#include "prof_cmocka.h"
 #include <stdlib.h>
 
 #include "xmpp/chat_session.h"
