@@ -1,9 +1,6 @@
 #include "xmpp/resource.h"
 #include "common.h"
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
+#include "prof_cmocka.h"
 #include <stdlib.h>
 
 void
@@ -543,10 +540,10 @@ prof_occurrences_of_large_message_tests(void** state)
      */
     const size_t haystack_sz = 1024;
     size_t haystack_cur = 0;
-    char* haystack = malloc(haystack_sz);
+    auto_char char* haystack = malloc(haystack_sz);
     const char needle[] = "needle ";
     while (haystack_sz - haystack_cur > sizeof(needle)) {
-        memcpy(&haystack[haystack_cur], needle, sizeof(needle) - 1);
+        memcpy(&haystack[haystack_cur], needle, sizeof(needle));
         expected = g_slist_append(expected, GINT_TO_POINTER(haystack_cur));
         haystack_cur += sizeof(needle) - 1;
     }
